This file is not an actual template, but used to add some
texts in extensions to sphinx.pot file.

In extensions/pyspecific.py:

{% trans %}CPython implementation detail:{% endtrans %}
{% trans %}Deprecated since version {deprecated}, will be removed in version {removed}{% endtrans %}
