;-------------------------------------------------------------------------------------------------
; <copyright file="wixstdba.def" company="Outercurve Foundation">
;   Copyright (c) 2004, Outercurve Foundation.
;   This software is released under Microsoft Reciprocal License (MS-RL).
;   The license and further copyright text can be found in the file
;   LICENSE.TXT at the root directory of the distribution.
; </copyright>
; 
; <summary>
;    WiX Standard Bootstrapper Application DLL entry points.
; </summary>
;-------------------------------------------------------------------------------------------------

EXPORTS
    BootstrapperApplicationCreate
    BootstrapperApplicationDestroy
    MbaPrereqBootstrapperApplicationCreate
    MbaPrereqBootstrapperApplicationDestroy
