/* File generated by Tools/ssl/make_ssl_data.py */
/* Generated on 2015-01-17T20:33:43.377453 */

static struct py_ssl_library_code library_codes[] = {
    {"PEM", ERR_LIB_PEM},
    {"SSL", ERR_LIB_SSL},
    {"X509", ERR_LIB_X509},
    { NULL }
};

static struct py_ssl_error_code error_codes[] = {
  #ifdef PEM_R_BAD_BASE64_DECODE
    {"BAD_BASE64_DECODE", ERR_LIB_PEM, PEM_R_BAD_BASE64_DECODE},
  #else
    {"BAD_BASE64_DECODE", ERR_LIB_PEM, 100},
  #endif
  #ifdef PEM_R_BAD_DECRYPT
    {"BAD_DECRYPT", ERR_LIB_PEM, PEM_R_BAD_DECRYPT},
  #else
    {"BAD_DECRYPT", ERR_LIB_PEM, 101},
  #endif
  #ifdef PEM_R_BAD_END_LINE
    {"BAD_END_LINE", ERR_LIB_PEM, PEM_R_BAD_END_LINE},
  #else
    {"BAD_END_LINE", ERR_LIB_PEM, 102},
  #endif
  #ifdef PEM_R_BAD_IV_CHARS
    {"BAD_IV_CHARS", ERR_LIB_PEM, PEM_R_BAD_IV_CHARS},
  #else
    {"BAD_IV_CHARS", ERR_LIB_PEM, 103},
  #endif
  #ifdef PEM_R_BAD_MAGIC_NUMBER
    {"BAD_MAGIC_NUMBER", ERR_LIB_PEM, PEM_R_BAD_MAGIC_NUMBER},
  #else
    {"BAD_MAGIC_NUMBER", ERR_LIB_PEM, 116},
  #endif
  #ifdef PEM_R_BAD_PASSWORD_READ
    {"BAD_PASSWORD_READ", ERR_LIB_PEM, PEM_R_BAD_PASSWORD_READ},
  #else
    {"BAD_PASSWORD_READ", ERR_LIB_PEM, 104},
  #endif
  #ifdef PEM_R_BAD_VERSION_NUMBER
    {"BAD_VERSION_NUMBER", ERR_LIB_PEM, PEM_R_BAD_VERSION_NUMBER},
  #else
    {"BAD_VERSION_NUMBER", ERR_LIB_PEM, 117},
  #endif
  #ifdef PEM_R_BIO_WRITE_FAILURE
    {"BIO_WRITE_FAILURE", ERR_LIB_PEM, PEM_R_BIO_WRITE_FAILURE},
  #else
    {"BIO_WRITE_FAILURE", ERR_LIB_PEM, 118},
  #endif
  #ifdef PEM_R_CIPHER_IS_NULL
    {"CIPHER_IS_NULL", ERR_LIB_PEM, PEM_R_CIPHER_IS_NULL},
  #else
    {"CIPHER_IS_NULL", ERR_LIB_PEM, 127},
  #endif
  #ifdef PEM_R_ERROR_CONVERTING_PRIVATE_KEY
    {"ERROR_CONVERTING_PRIVATE_KEY", ERR_LIB_PEM, PEM_R_ERROR_CONVERTING_PRIVATE_KEY},
  #else
    {"ERROR_CONVERTING_PRIVATE_KEY", ERR_LIB_PEM, 115},
  #endif
  #ifdef PEM_R_EXPECTING_PRIVATE_KEY_BLOB
    {"EXPECTING_PRIVATE_KEY_BLOB", ERR_LIB_PEM, PEM_R_EXPECTING_PRIVATE_KEY_BLOB},
  #else
    {"EXPECTING_PRIVATE_KEY_BLOB", ERR_LIB_PEM, 119},
  #endif
  #ifdef PEM_R_EXPECTING_PUBLIC_KEY_BLOB
    {"EXPECTING_PUBLIC_KEY_BLOB", ERR_LIB_PEM, PEM_R_EXPECTING_PUBLIC_KEY_BLOB},
  #else
    {"EXPECTING_PUBLIC_KEY_BLOB", ERR_LIB_PEM, 120},
  #endif
  #ifdef PEM_R_INCONSISTENT_HEADER
    {"INCONSISTENT_HEADER", ERR_LIB_PEM, PEM_R_INCONSISTENT_HEADER},
  #else
    {"INCONSISTENT_HEADER", ERR_LIB_PEM, 121},
  #endif
  #ifdef PEM_R_KEYBLOB_HEADER_PARSE_ERROR
    {"KEYBLOB_HEADER_PARSE_ERROR", ERR_LIB_PEM, PEM_R_KEYBLOB_HEADER_PARSE_ERROR},
  #else
    {"KEYBLOB_HEADER_PARSE_ERROR", ERR_LIB_PEM, 122},
  #endif
  #ifdef PEM_R_KEYBLOB_TOO_SHORT
    {"KEYBLOB_TOO_SHORT", ERR_LIB_PEM, PEM_R_KEYBLOB_TOO_SHORT},
  #else
    {"KEYBLOB_TOO_SHORT", ERR_LIB_PEM, 123},
  #endif
  #ifdef PEM_R_NOT_DEK_INFO
    {"NOT_DEK_INFO", ERR_LIB_PEM, PEM_R_NOT_DEK_INFO},
  #else
    {"NOT_DEK_INFO", ERR_LIB_PEM, 105},
  #endif
  #ifdef PEM_R_NOT_ENCRYPTED
    {"NOT_ENCRYPTED", ERR_LIB_PEM, PEM_R_NOT_ENCRYPTED},
  #else
    {"NOT_ENCRYPTED", ERR_LIB_PEM, 106},
  #endif
  #ifdef PEM_R_NOT_PROC_TYPE
    {"NOT_PROC_TYPE", ERR_LIB_PEM, PEM_R_NOT_PROC_TYPE},
  #else
    {"NOT_PROC_TYPE", ERR_LIB_PEM, 107},
  #endif
  #ifdef PEM_R_NO_START_LINE
    {"NO_START_LINE", ERR_LIB_PEM, PEM_R_NO_START_LINE},
  #else
    {"NO_START_LINE", ERR_LIB_PEM, 108},
  #endif
  #ifdef PEM_R_PROBLEMS_GETTING_PASSWORD
    {"PROBLEMS_GETTING_PASSWORD", ERR_LIB_PEM, PEM_R_PROBLEMS_GETTING_PASSWORD},
  #else
    {"PROBLEMS_GETTING_PASSWORD", ERR_LIB_PEM, 109},
  #endif
  #ifdef PEM_R_PUBLIC_KEY_NO_RSA
    {"PUBLIC_KEY_NO_RSA", ERR_LIB_PEM, PEM_R_PUBLIC_KEY_NO_RSA},
  #else
    {"PUBLIC_KEY_NO_RSA", ERR_LIB_PEM, 110},
  #endif
  #ifdef PEM_R_PVK_DATA_TOO_SHORT
    {"PVK_DATA_TOO_SHORT", ERR_LIB_PEM, PEM_R_PVK_DATA_TOO_SHORT},
  #else
    {"PVK_DATA_TOO_SHORT", ERR_LIB_PEM, 124},
  #endif
  #ifdef PEM_R_PVK_TOO_SHORT
    {"PVK_TOO_SHORT", ERR_LIB_PEM, PEM_R_PVK_TOO_SHORT},
  #else
    {"PVK_TOO_SHORT", ERR_LIB_PEM, 125},
  #endif
  #ifdef PEM_R_READ_KEY
    {"READ_KEY", ERR_LIB_PEM, PEM_R_READ_KEY},
  #else
    {"READ_KEY", ERR_LIB_PEM, 111},
  #endif
  #ifdef PEM_R_SHORT_HEADER
    {"SHORT_HEADER", ERR_LIB_PEM, PEM_R_SHORT_HEADER},
  #else
    {"SHORT_HEADER", ERR_LIB_PEM, 112},
  #endif
  #ifdef PEM_R_UNSUPPORTED_CIPHER
    {"UNSUPPORTED_CIPHER", ERR_LIB_PEM, PEM_R_UNSUPPORTED_CIPHER},
  #else
    {"UNSUPPORTED_CIPHER", ERR_LIB_PEM, 113},
  #endif
  #ifdef PEM_R_UNSUPPORTED_ENCRYPTION
    {"UNSUPPORTED_ENCRYPTION", ERR_LIB_PEM, PEM_R_UNSUPPORTED_ENCRYPTION},
  #else
    {"UNSUPPORTED_ENCRYPTION", ERR_LIB_PEM, 114},
  #endif
  #ifdef PEM_R_UNSUPPORTED_KEY_COMPONENTS
    {"UNSUPPORTED_KEY_COMPONENTS", ERR_LIB_PEM, PEM_R_UNSUPPORTED_KEY_COMPONENTS},
  #else
    {"UNSUPPORTED_KEY_COMPONENTS", ERR_LIB_PEM, 126},
  #endif
  #ifdef SSL_R_APP_DATA_IN_HANDSHAKE
    {"APP_DATA_IN_HANDSHAKE", ERR_LIB_SSL, SSL_R_APP_DATA_IN_HANDSHAKE},
  #else
    {"APP_DATA_IN_HANDSHAKE", ERR_LIB_SSL, 100},
  #endif
  #ifdef SSL_R_ATTEMPT_TO_REUSE_SESSION_IN_DIFFERENT_CONTEXT
    {"ATTEMPT_TO_REUSE_SESSION_IN_DIFFERENT_CONTEXT", ERR_LIB_SSL, SSL_R_ATTEMPT_TO_REUSE_SESSION_IN_DIFFERENT_CONTEXT},
  #else
    {"ATTEMPT_TO_REUSE_SESSION_IN_DIFFERENT_CONTEXT", ERR_LIB_SSL, 272},
  #endif
  #ifdef SSL_R_BAD_ALERT_RECORD
    {"BAD_ALERT_RECORD", ERR_LIB_SSL, SSL_R_BAD_ALERT_RECORD},
  #else
    {"BAD_ALERT_RECORD", ERR_LIB_SSL, 101},
  #endif
  #ifdef SSL_R_BAD_AUTHENTICATION_TYPE
    {"BAD_AUTHENTICATION_TYPE", ERR_LIB_SSL, SSL_R_BAD_AUTHENTICATION_TYPE},
  #else
    {"BAD_AUTHENTICATION_TYPE", ERR_LIB_SSL, 102},
  #endif
  #ifdef SSL_R_BAD_CHANGE_CIPHER_SPEC
    {"BAD_CHANGE_CIPHER_SPEC", ERR_LIB_SSL, SSL_R_BAD_CHANGE_CIPHER_SPEC},
  #else
    {"BAD_CHANGE_CIPHER_SPEC", ERR_LIB_SSL, 103},
  #endif
  #ifdef SSL_R_BAD_CHECKSUM
    {"BAD_CHECKSUM", ERR_LIB_SSL, SSL_R_BAD_CHECKSUM},
  #else
    {"BAD_CHECKSUM", ERR_LIB_SSL, 104},
  #endif
  #ifdef SSL_R_BAD_DATA
    {"BAD_DATA", ERR_LIB_SSL, SSL_R_BAD_DATA},
  #else
    {"BAD_DATA", ERR_LIB_SSL, 390},
  #endif
  #ifdef SSL_R_BAD_DATA_RETURNED_BY_CALLBACK
    {"BAD_DATA_RETURNED_BY_CALLBACK", ERR_LIB_SSL, SSL_R_BAD_DATA_RETURNED_BY_CALLBACK},
  #else
    {"BAD_DATA_RETURNED_BY_CALLBACK", ERR_LIB_SSL, 106},
  #endif
  #ifdef SSL_R_BAD_DECOMPRESSION
    {"BAD_DECOMPRESSION", ERR_LIB_SSL, SSL_R_BAD_DECOMPRESSION},
  #else
    {"BAD_DECOMPRESSION", ERR_LIB_SSL, 107},
  #endif
  #ifdef SSL_R_BAD_DH_G_LENGTH
    {"BAD_DH_G_LENGTH", ERR_LIB_SSL, SSL_R_BAD_DH_G_LENGTH},
  #else
    {"BAD_DH_G_LENGTH", ERR_LIB_SSL, 108},
  #endif
  #ifdef SSL_R_BAD_DH_PUB_KEY_LENGTH
    {"BAD_DH_PUB_KEY_LENGTH", ERR_LIB_SSL, SSL_R_BAD_DH_PUB_KEY_LENGTH},
  #else
    {"BAD_DH_PUB_KEY_LENGTH", ERR_LIB_SSL, 109},
  #endif
  #ifdef SSL_R_BAD_DH_P_LENGTH
    {"BAD_DH_P_LENGTH", ERR_LIB_SSL, SSL_R_BAD_DH_P_LENGTH},
  #else
    {"BAD_DH_P_LENGTH", ERR_LIB_SSL, 110},
  #endif
  #ifdef SSL_R_BAD_DIGEST_LENGTH
    {"BAD_DIGEST_LENGTH", ERR_LIB_SSL, SSL_R_BAD_DIGEST_LENGTH},
  #else
    {"BAD_DIGEST_LENGTH", ERR_LIB_SSL, 111},
  #endif
  #ifdef SSL_R_BAD_DSA_SIGNATURE
    {"BAD_DSA_SIGNATURE", ERR_LIB_SSL, SSL_R_BAD_DSA_SIGNATURE},
  #else
    {"BAD_DSA_SIGNATURE", ERR_LIB_SSL, 112},
  #endif
  #ifdef SSL_R_BAD_ECC_CERT
    {"BAD_ECC_CERT", ERR_LIB_SSL, SSL_R_BAD_ECC_CERT},
  #else
    {"BAD_ECC_CERT", ERR_LIB_SSL, 304},
  #endif
  #ifdef SSL_R_BAD_ECDSA_SIGNATURE
    {"BAD_ECDSA_SIGNATURE", ERR_LIB_SSL, SSL_R_BAD_ECDSA_SIGNATURE},
  #else
    {"BAD_ECDSA_SIGNATURE", ERR_LIB_SSL, 305},
  #endif
  #ifdef SSL_R_BAD_ECPOINT
    {"BAD_ECPOINT", ERR_LIB_SSL, SSL_R_BAD_ECPOINT},
  #else
    {"BAD_ECPOINT", ERR_LIB_SSL, 306},
  #endif
  #ifdef SSL_R_BAD_HANDSHAKE_LENGTH
    {"BAD_HANDSHAKE_LENGTH", ERR_LIB_SSL, SSL_R_BAD_HANDSHAKE_LENGTH},
  #else
    {"BAD_HANDSHAKE_LENGTH", ERR_LIB_SSL, 332},
  #endif
  #ifdef SSL_R_BAD_HELLO_REQUEST
    {"BAD_HELLO_REQUEST", ERR_LIB_SSL, SSL_R_BAD_HELLO_REQUEST},
  #else
    {"BAD_HELLO_REQUEST", ERR_LIB_SSL, 105},
  #endif
  #ifdef SSL_R_BAD_LENGTH
    {"BAD_LENGTH", ERR_LIB_SSL, SSL_R_BAD_LENGTH},
  #else
    {"BAD_LENGTH", ERR_LIB_SSL, 271},
  #endif
  #ifdef SSL_R_BAD_MAC_DECODE
    {"BAD_MAC_DECODE", ERR_LIB_SSL, SSL_R_BAD_MAC_DECODE},
  #else
    {"BAD_MAC_DECODE", ERR_LIB_SSL, 113},
  #endif
  #ifdef SSL_R_BAD_MAC_LENGTH
    {"BAD_MAC_LENGTH", ERR_LIB_SSL, SSL_R_BAD_MAC_LENGTH},
  #else
    {"BAD_MAC_LENGTH", ERR_LIB_SSL, 333},
  #endif
  #ifdef SSL_R_BAD_MESSAGE_TYPE
    {"BAD_MESSAGE_TYPE", ERR_LIB_SSL, SSL_R_BAD_MESSAGE_TYPE},
  #else
    {"BAD_MESSAGE_TYPE", ERR_LIB_SSL, 114},
  #endif
  #ifdef SSL_R_BAD_PACKET_LENGTH
    {"BAD_PACKET_LENGTH", ERR_LIB_SSL, SSL_R_BAD_PACKET_LENGTH},
  #else
    {"BAD_PACKET_LENGTH", ERR_LIB_SSL, 115},
  #endif
  #ifdef SSL_R_BAD_PROTOCOL_VERSION_NUMBER
    {"BAD_PROTOCOL_VERSION_NUMBER", ERR_LIB_SSL, SSL_R_BAD_PROTOCOL_VERSION_NUMBER},
  #else
    {"BAD_PROTOCOL_VERSION_NUMBER", ERR_LIB_SSL, 116},
  #endif
  #ifdef SSL_R_BAD_PSK_IDENTITY_HINT_LENGTH
    {"BAD_PSK_IDENTITY_HINT_LENGTH", ERR_LIB_SSL, SSL_R_BAD_PSK_IDENTITY_HINT_LENGTH},
  #else
    {"BAD_PSK_IDENTITY_HINT_LENGTH", ERR_LIB_SSL, 316},
  #endif
  #ifdef SSL_R_BAD_RESPONSE_ARGUMENT
    {"BAD_RESPONSE_ARGUMENT", ERR_LIB_SSL, SSL_R_BAD_RESPONSE_ARGUMENT},
  #else
    {"BAD_RESPONSE_ARGUMENT", ERR_LIB_SSL, 117},
  #endif
  #ifdef SSL_R_BAD_RSA_DECRYPT
    {"BAD_RSA_DECRYPT", ERR_LIB_SSL, SSL_R_BAD_RSA_DECRYPT},
  #else
    {"BAD_RSA_DECRYPT", ERR_LIB_SSL, 118},
  #endif
  #ifdef SSL_R_BAD_RSA_ENCRYPT
    {"BAD_RSA_ENCRYPT", ERR_LIB_SSL, SSL_R_BAD_RSA_ENCRYPT},
  #else
    {"BAD_RSA_ENCRYPT", ERR_LIB_SSL, 119},
  #endif
  #ifdef SSL_R_BAD_RSA_E_LENGTH
    {"BAD_RSA_E_LENGTH", ERR_LIB_SSL, SSL_R_BAD_RSA_E_LENGTH},
  #else
    {"BAD_RSA_E_LENGTH", ERR_LIB_SSL, 120},
  #endif
  #ifdef SSL_R_BAD_RSA_MODULUS_LENGTH
    {"BAD_RSA_MODULUS_LENGTH", ERR_LIB_SSL, SSL_R_BAD_RSA_MODULUS_LENGTH},
  #else
    {"BAD_RSA_MODULUS_LENGTH", ERR_LIB_SSL, 121},
  #endif
  #ifdef SSL_R_BAD_RSA_SIGNATURE
    {"BAD_RSA_SIGNATURE", ERR_LIB_SSL, SSL_R_BAD_RSA_SIGNATURE},
  #else
    {"BAD_RSA_SIGNATURE", ERR_LIB_SSL, 122},
  #endif
  #ifdef SSL_R_BAD_SIGNATURE
    {"BAD_SIGNATURE", ERR_LIB_SSL, SSL_R_BAD_SIGNATURE},
  #else
    {"BAD_SIGNATURE", ERR_LIB_SSL, 123},
  #endif
  #ifdef SSL_R_BAD_SRP_A_LENGTH
    {"BAD_SRP_A_LENGTH", ERR_LIB_SSL, SSL_R_BAD_SRP_A_LENGTH},
  #else
    {"BAD_SRP_A_LENGTH", ERR_LIB_SSL, 347},
  #endif
  #ifdef SSL_R_BAD_SRP_B_LENGTH
    {"BAD_SRP_B_LENGTH", ERR_LIB_SSL, SSL_R_BAD_SRP_B_LENGTH},
  #else
    {"BAD_SRP_B_LENGTH", ERR_LIB_SSL, 348},
  #endif
  #ifdef SSL_R_BAD_SRP_G_LENGTH
    {"BAD_SRP_G_LENGTH", ERR_LIB_SSL, SSL_R_BAD_SRP_G_LENGTH},
  #else
    {"BAD_SRP_G_LENGTH", ERR_LIB_SSL, 349},
  #endif
  #ifdef SSL_R_BAD_SRP_N_LENGTH
    {"BAD_SRP_N_LENGTH", ERR_LIB_SSL, SSL_R_BAD_SRP_N_LENGTH},
  #else
    {"BAD_SRP_N_LENGTH", ERR_LIB_SSL, 350},
  #endif
  #ifdef SSL_R_BAD_SRP_PARAMETERS
    {"BAD_SRP_PARAMETERS", ERR_LIB_SSL, SSL_R_BAD_SRP_PARAMETERS},
  #else
    {"BAD_SRP_PARAMETERS", ERR_LIB_SSL, 371},
  #endif
  #ifdef SSL_R_BAD_SRP_S_LENGTH
    {"BAD_SRP_S_LENGTH", ERR_LIB_SSL, SSL_R_BAD_SRP_S_LENGTH},
  #else
    {"BAD_SRP_S_LENGTH", ERR_LIB_SSL, 351},
  #endif
  #ifdef SSL_R_BAD_SRTP_MKI_VALUE
    {"BAD_SRTP_MKI_VALUE", ERR_LIB_SSL, SSL_R_BAD_SRTP_MKI_VALUE},
  #else
    {"BAD_SRTP_MKI_VALUE", ERR_LIB_SSL, 352},
  #endif
  #ifdef SSL_R_BAD_SRTP_PROTECTION_PROFILE_LIST
    {"BAD_SRTP_PROTECTION_PROFILE_LIST", ERR_LIB_SSL, SSL_R_BAD_SRTP_PROTECTION_PROFILE_LIST},
  #else
    {"BAD_SRTP_PROTECTION_PROFILE_LIST", ERR_LIB_SSL, 353},
  #endif
  #ifdef SSL_R_BAD_SSL_FILETYPE
    {"BAD_SSL_FILETYPE", ERR_LIB_SSL, SSL_R_BAD_SSL_FILETYPE},
  #else
    {"BAD_SSL_FILETYPE", ERR_LIB_SSL, 124},
  #endif
  #ifdef SSL_R_BAD_SSL_SESSION_ID_LENGTH
    {"BAD_SSL_SESSION_ID_LENGTH", ERR_LIB_SSL, SSL_R_BAD_SSL_SESSION_ID_LENGTH},
  #else
    {"BAD_SSL_SESSION_ID_LENGTH", ERR_LIB_SSL, 125},
  #endif
  #ifdef SSL_R_BAD_STATE
    {"BAD_STATE", ERR_LIB_SSL, SSL_R_BAD_STATE},
  #else
    {"BAD_STATE", ERR_LIB_SSL, 126},
  #endif
  #ifdef SSL_R_BAD_VALUE
    {"BAD_VALUE", ERR_LIB_SSL, SSL_R_BAD_VALUE},
  #else
    {"BAD_VALUE", ERR_LIB_SSL, 384},
  #endif
  #ifdef SSL_R_BAD_WRITE_RETRY
    {"BAD_WRITE_RETRY", ERR_LIB_SSL, SSL_R_BAD_WRITE_RETRY},
  #else
    {"BAD_WRITE_RETRY", ERR_LIB_SSL, 127},
  #endif
  #ifdef SSL_R_BIO_NOT_SET
    {"BIO_NOT_SET", ERR_LIB_SSL, SSL_R_BIO_NOT_SET},
  #else
    {"BIO_NOT_SET", ERR_LIB_SSL, 128},
  #endif
  #ifdef SSL_R_BLOCK_CIPHER_PAD_IS_WRONG
    {"BLOCK_CIPHER_PAD_IS_WRONG", ERR_LIB_SSL, SSL_R_BLOCK_CIPHER_PAD_IS_WRONG},
  #else
    {"BLOCK_CIPHER_PAD_IS_WRONG", ERR_LIB_SSL, 129},
  #endif
  #ifdef SSL_R_BN_LIB
    {"BN_LIB", ERR_LIB_SSL, SSL_R_BN_LIB},
  #else
    {"BN_LIB", ERR_LIB_SSL, 130},
  #endif
  #ifdef SSL_R_CA_DN_LENGTH_MISMATCH
    {"CA_DN_LENGTH_MISMATCH", ERR_LIB_SSL, SSL_R_CA_DN_LENGTH_MISMATCH},
  #else
    {"CA_DN_LENGTH_MISMATCH", ERR_LIB_SSL, 131},
  #endif
  #ifdef SSL_R_CA_DN_TOO_LONG
    {"CA_DN_TOO_LONG", ERR_LIB_SSL, SSL_R_CA_DN_TOO_LONG},
  #else
    {"CA_DN_TOO_LONG", ERR_LIB_SSL, 132},
  #endif
  #ifdef SSL_R_CA_KEY_TOO_SMALL
    {"CA_KEY_TOO_SMALL", ERR_LIB_SSL, SSL_R_CA_KEY_TOO_SMALL},
  #else
    {"CA_KEY_TOO_SMALL", ERR_LIB_SSL, 397},
  #endif
  #ifdef SSL_R_CA_MD_TOO_WEAK
    {"CA_MD_TOO_WEAK", ERR_LIB_SSL, SSL_R_CA_MD_TOO_WEAK},
  #else
    {"CA_MD_TOO_WEAK", ERR_LIB_SSL, 398},
  #endif
  #ifdef SSL_R_CCS_RECEIVED_EARLY
    {"CCS_RECEIVED_EARLY", ERR_LIB_SSL, SSL_R_CCS_RECEIVED_EARLY},
  #else
    {"CCS_RECEIVED_EARLY", ERR_LIB_SSL, 133},
  #endif
  #ifdef SSL_R_CERTIFICATE_VERIFY_FAILED
    {"CERTIFICATE_VERIFY_FAILED", ERR_LIB_SSL, SSL_R_CERTIFICATE_VERIFY_FAILED},
  #else
    {"CERTIFICATE_VERIFY_FAILED", ERR_LIB_SSL, 134},
  #endif
  #ifdef SSL_R_CERT_CB_ERROR
    {"CERT_CB_ERROR", ERR_LIB_SSL, SSL_R_CERT_CB_ERROR},
  #else
    {"CERT_CB_ERROR", ERR_LIB_SSL, 377},
  #endif
  #ifdef SSL_R_CERT_LENGTH_MISMATCH
    {"CERT_LENGTH_MISMATCH", ERR_LIB_SSL, SSL_R_CERT_LENGTH_MISMATCH},
  #else
    {"CERT_LENGTH_MISMATCH", ERR_LIB_SSL, 135},
  #endif
  #ifdef SSL_R_CHALLENGE_IS_DIFFERENT
    {"CHALLENGE_IS_DIFFERENT", ERR_LIB_SSL, SSL_R_CHALLENGE_IS_DIFFERENT},
  #else
    {"CHALLENGE_IS_DIFFERENT", ERR_LIB_SSL, 136},
  #endif
  #ifdef SSL_R_CIPHER_CODE_WRONG_LENGTH
    {"CIPHER_CODE_WRONG_LENGTH", ERR_LIB_SSL, SSL_R_CIPHER_CODE_WRONG_LENGTH},
  #else
    {"CIPHER_CODE_WRONG_LENGTH", ERR_LIB_SSL, 137},
  #endif
  #ifdef SSL_R_CIPHER_OR_HASH_UNAVAILABLE
    {"CIPHER_OR_HASH_UNAVAILABLE", ERR_LIB_SSL, SSL_R_CIPHER_OR_HASH_UNAVAILABLE},
  #else
    {"CIPHER_OR_HASH_UNAVAILABLE", ERR_LIB_SSL, 138},
  #endif
  #ifdef SSL_R_CIPHER_TABLE_SRC_ERROR
    {"CIPHER_TABLE_SRC_ERROR", ERR_LIB_SSL, SSL_R_CIPHER_TABLE_SRC_ERROR},
  #else
    {"CIPHER_TABLE_SRC_ERROR", ERR_LIB_SSL, 139},
  #endif
  #ifdef SSL_R_CLIENTHELLO_TLSEXT
    {"CLIENTHELLO_TLSEXT", ERR_LIB_SSL, SSL_R_CLIENTHELLO_TLSEXT},
  #else
    {"CLIENTHELLO_TLSEXT", ERR_LIB_SSL, 226},
  #endif
  #ifdef SSL_R_COMPRESSED_LENGTH_TOO_LONG
    {"COMPRESSED_LENGTH_TOO_LONG", ERR_LIB_SSL, SSL_R_COMPRESSED_LENGTH_TOO_LONG},
  #else
    {"COMPRESSED_LENGTH_TOO_LONG", ERR_LIB_SSL, 140},
  #endif
  #ifdef SSL_R_COMPRESSION_DISABLED
    {"COMPRESSION_DISABLED", ERR_LIB_SSL, SSL_R_COMPRESSION_DISABLED},
  #else
    {"COMPRESSION_DISABLED", ERR_LIB_SSL, 343},
  #endif
  #ifdef SSL_R_COMPRESSION_FAILURE
    {"COMPRESSION_FAILURE", ERR_LIB_SSL, SSL_R_COMPRESSION_FAILURE},
  #else
    {"COMPRESSION_FAILURE", ERR_LIB_SSL, 141},
  #endif
  #ifdef SSL_R_COMPRESSION_ID_NOT_WITHIN_PRIVATE_RANGE
    {"COMPRESSION_ID_NOT_WITHIN_PRIVATE_RANGE", ERR_LIB_SSL, SSL_R_COMPRESSION_ID_NOT_WITHIN_PRIVATE_RANGE},
  #else
    {"COMPRESSION_ID_NOT_WITHIN_PRIVATE_RANGE", ERR_LIB_SSL, 307},
  #endif
  #ifdef SSL_R_COMPRESSION_LIBRARY_ERROR
    {"COMPRESSION_LIBRARY_ERROR", ERR_LIB_SSL, SSL_R_COMPRESSION_LIBRARY_ERROR},
  #else
    {"COMPRESSION_LIBRARY_ERROR", ERR_LIB_SSL, 142},
  #endif
  #ifdef SSL_R_CONNECTION_ID_IS_DIFFERENT
    {"CONNECTION_ID_IS_DIFFERENT", ERR_LIB_SSL, SSL_R_CONNECTION_ID_IS_DIFFERENT},
  #else
    {"CONNECTION_ID_IS_DIFFERENT", ERR_LIB_SSL, 143},
  #endif
  #ifdef SSL_R_CONNECTION_TYPE_NOT_SET
    {"CONNECTION_TYPE_NOT_SET", ERR_LIB_SSL, SSL_R_CONNECTION_TYPE_NOT_SET},
  #else
    {"CONNECTION_TYPE_NOT_SET", ERR_LIB_SSL, 144},
  #endif
  #ifdef SSL_R_COOKIE_MISMATCH
    {"COOKIE_MISMATCH", ERR_LIB_SSL, SSL_R_COOKIE_MISMATCH},
  #else
    {"COOKIE_MISMATCH", ERR_LIB_SSL, 308},
  #endif
  #ifdef SSL_R_DATA_BETWEEN_CCS_AND_FINISHED
    {"DATA_BETWEEN_CCS_AND_FINISHED", ERR_LIB_SSL, SSL_R_DATA_BETWEEN_CCS_AND_FINISHED},
  #else
    {"DATA_BETWEEN_CCS_AND_FINISHED", ERR_LIB_SSL, 145},
  #endif
  #ifdef SSL_R_DATA_LENGTH_TOO_LONG
    {"DATA_LENGTH_TOO_LONG", ERR_LIB_SSL, SSL_R_DATA_LENGTH_TOO_LONG},
  #else
    {"DATA_LENGTH_TOO_LONG", ERR_LIB_SSL, 146},
  #endif
  #ifdef SSL_R_DECRYPTION_FAILED
    {"DECRYPTION_FAILED", ERR_LIB_SSL, SSL_R_DECRYPTION_FAILED},
  #else
    {"DECRYPTION_FAILED", ERR_LIB_SSL, 147},
  #endif
  #ifdef SSL_R_DECRYPTION_FAILED_OR_BAD_RECORD_MAC
    {"DECRYPTION_FAILED_OR_BAD_RECORD_MAC", ERR_LIB_SSL, SSL_R_DECRYPTION_FAILED_OR_BAD_RECORD_MAC},
  #else
    {"DECRYPTION_FAILED_OR_BAD_RECORD_MAC", ERR_LIB_SSL, 281},
  #endif
  #ifdef SSL_R_DH_KEY_TOO_SMALL
    {"DH_KEY_TOO_SMALL", ERR_LIB_SSL, SSL_R_DH_KEY_TOO_SMALL},
  #else
    {"DH_KEY_TOO_SMALL", ERR_LIB_SSL, 394},
  #endif
  #ifdef SSL_R_DH_PUBLIC_VALUE_LENGTH_IS_WRONG
    {"DH_PUBLIC_VALUE_LENGTH_IS_WRONG", ERR_LIB_SSL, SSL_R_DH_PUBLIC_VALUE_LENGTH_IS_WRONG},
  #else
    {"DH_PUBLIC_VALUE_LENGTH_IS_WRONG", ERR_LIB_SSL, 148},
  #endif
  #ifdef SSL_R_DIGEST_CHECK_FAILED
    {"DIGEST_CHECK_FAILED", ERR_LIB_SSL, SSL_R_DIGEST_CHECK_FAILED},
  #else
    {"DIGEST_CHECK_FAILED", ERR_LIB_SSL, 149},
  #endif
  #ifdef SSL_R_DTLS_MESSAGE_TOO_BIG
    {"DTLS_MESSAGE_TOO_BIG", ERR_LIB_SSL, SSL_R_DTLS_MESSAGE_TOO_BIG},
  #else
    {"DTLS_MESSAGE_TOO_BIG", ERR_LIB_SSL, 334},
  #endif
  #ifdef SSL_R_DUPLICATE_COMPRESSION_ID
    {"DUPLICATE_COMPRESSION_ID", ERR_LIB_SSL, SSL_R_DUPLICATE_COMPRESSION_ID},
  #else
    {"DUPLICATE_COMPRESSION_ID", ERR_LIB_SSL, 309},
  #endif
  #ifdef SSL_R_ECC_CERT_NOT_FOR_KEY_AGREEMENT
    {"ECC_CERT_NOT_FOR_KEY_AGREEMENT", ERR_LIB_SSL, SSL_R_ECC_CERT_NOT_FOR_KEY_AGREEMENT},
  #else
    {"ECC_CERT_NOT_FOR_KEY_AGREEMENT", ERR_LIB_SSL, 317},
  #endif
  #ifdef SSL_R_ECC_CERT_NOT_FOR_SIGNING
    {"ECC_CERT_NOT_FOR_SIGNING", ERR_LIB_SSL, SSL_R_ECC_CERT_NOT_FOR_SIGNING},
  #else
    {"ECC_CERT_NOT_FOR_SIGNING", ERR_LIB_SSL, 318},
  #endif
  #ifdef SSL_R_ECC_CERT_SHOULD_HAVE_RSA_SIGNATURE
    {"ECC_CERT_SHOULD_HAVE_RSA_SIGNATURE", ERR_LIB_SSL, SSL_R_ECC_CERT_SHOULD_HAVE_RSA_SIGNATURE},
  #else
    {"ECC_CERT_SHOULD_HAVE_RSA_SIGNATURE", ERR_LIB_SSL, 322},
  #endif
  #ifdef SSL_R_ECC_CERT_SHOULD_HAVE_SHA1_SIGNATURE
    {"ECC_CERT_SHOULD_HAVE_SHA1_SIGNATURE", ERR_LIB_SSL, SSL_R_ECC_CERT_SHOULD_HAVE_SHA1_SIGNATURE},
  #else
    {"ECC_CERT_SHOULD_HAVE_SHA1_SIGNATURE", ERR_LIB_SSL, 323},
  #endif
  #ifdef SSL_R_ECDH_REQUIRED_FOR_SUITEB_MODE
    {"ECDH_REQUIRED_FOR_SUITEB_MODE", ERR_LIB_SSL, SSL_R_ECDH_REQUIRED_FOR_SUITEB_MODE},
  #else
    {"ECDH_REQUIRED_FOR_SUITEB_MODE", ERR_LIB_SSL, 374},
  #endif
  #ifdef SSL_R_ECGROUP_TOO_LARGE_FOR_CIPHER
    {"ECGROUP_TOO_LARGE_FOR_CIPHER", ERR_LIB_SSL, SSL_R_ECGROUP_TOO_LARGE_FOR_CIPHER},
  #else
    {"ECGROUP_TOO_LARGE_FOR_CIPHER", ERR_LIB_SSL, 310},
  #endif
  #ifdef SSL_R_EE_KEY_TOO_SMALL
    {"EE_KEY_TOO_SMALL", ERR_LIB_SSL, SSL_R_EE_KEY_TOO_SMALL},
  #else
    {"EE_KEY_TOO_SMALL", ERR_LIB_SSL, 399},
  #endif
  #ifdef SSL_R_EMPTY_SRTP_PROTECTION_PROFILE_LIST
    {"EMPTY_SRTP_PROTECTION_PROFILE_LIST", ERR_LIB_SSL, SSL_R_EMPTY_SRTP_PROTECTION_PROFILE_LIST},
  #else
    {"EMPTY_SRTP_PROTECTION_PROFILE_LIST", ERR_LIB_SSL, 354},
  #endif
  #ifdef SSL_R_ENCRYPTED_LENGTH_TOO_LONG
    {"ENCRYPTED_LENGTH_TOO_LONG", ERR_LIB_SSL, SSL_R_ENCRYPTED_LENGTH_TOO_LONG},
  #else
    {"ENCRYPTED_LENGTH_TOO_LONG", ERR_LIB_SSL, 150},
  #endif
  #ifdef SSL_R_ERROR_GENERATING_TMP_RSA_KEY
    {"ERROR_GENERATING_TMP_RSA_KEY", ERR_LIB_SSL, SSL_R_ERROR_GENERATING_TMP_RSA_KEY},
  #else
    {"ERROR_GENERATING_TMP_RSA_KEY", ERR_LIB_SSL, 282},
  #endif
  #ifdef SSL_R_ERROR_IN_RECEIVED_CIPHER_LIST
    {"ERROR_IN_RECEIVED_CIPHER_LIST", ERR_LIB_SSL, SSL_R_ERROR_IN_RECEIVED_CIPHER_LIST},
  #else
    {"ERROR_IN_RECEIVED_CIPHER_LIST", ERR_LIB_SSL, 151},
  #endif
  #ifdef SSL_R_EXCESSIVE_MESSAGE_SIZE
    {"EXCESSIVE_MESSAGE_SIZE", ERR_LIB_SSL, SSL_R_EXCESSIVE_MESSAGE_SIZE},
  #else
    {"EXCESSIVE_MESSAGE_SIZE", ERR_LIB_SSL, 152},
  #endif
  #ifdef SSL_R_EXTRA_DATA_IN_MESSAGE
    {"EXTRA_DATA_IN_MESSAGE", ERR_LIB_SSL, SSL_R_EXTRA_DATA_IN_MESSAGE},
  #else
    {"EXTRA_DATA_IN_MESSAGE", ERR_LIB_SSL, 153},
  #endif
  #ifdef SSL_R_GOT_A_FIN_BEFORE_A_CCS
    {"GOT_A_FIN_BEFORE_A_CCS", ERR_LIB_SSL, SSL_R_GOT_A_FIN_BEFORE_A_CCS},
  #else
    {"GOT_A_FIN_BEFORE_A_CCS", ERR_LIB_SSL, 154},
  #endif
  #ifdef SSL_R_GOT_NEXT_PROTO_BEFORE_A_CCS
    {"GOT_NEXT_PROTO_BEFORE_A_CCS", ERR_LIB_SSL, SSL_R_GOT_NEXT_PROTO_BEFORE_A_CCS},
  #else
    {"GOT_NEXT_PROTO_BEFORE_A_CCS", ERR_LIB_SSL, 355},
  #endif
  #ifdef SSL_R_GOT_NEXT_PROTO_WITHOUT_EXTENSION
    {"GOT_NEXT_PROTO_WITHOUT_EXTENSION", ERR_LIB_SSL, SSL_R_GOT_NEXT_PROTO_WITHOUT_EXTENSION},
  #else
    {"GOT_NEXT_PROTO_WITHOUT_EXTENSION", ERR_LIB_SSL, 356},
  #endif
  #ifdef SSL_R_HTTPS_PROXY_REQUEST
    {"HTTPS_PROXY_REQUEST", ERR_LIB_SSL, SSL_R_HTTPS_PROXY_REQUEST},
  #else
    {"HTTPS_PROXY_REQUEST", ERR_LIB_SSL, 155},
  #endif
  #ifdef SSL_R_HTTP_REQUEST
    {"HTTP_REQUEST", ERR_LIB_SSL, SSL_R_HTTP_REQUEST},
  #else
    {"HTTP_REQUEST", ERR_LIB_SSL, 156},
  #endif
  #ifdef SSL_R_ILLEGAL_PADDING
    {"ILLEGAL_PADDING", ERR_LIB_SSL, SSL_R_ILLEGAL_PADDING},
  #else
    {"ILLEGAL_PADDING", ERR_LIB_SSL, 283},
  #endif
  #ifdef SSL_R_ILLEGAL_SUITEB_DIGEST
    {"ILLEGAL_SUITEB_DIGEST", ERR_LIB_SSL, SSL_R_ILLEGAL_SUITEB_DIGEST},
  #else
    {"ILLEGAL_SUITEB_DIGEST", ERR_LIB_SSL, 380},
  #endif
  #ifdef SSL_R_INAPPROPRIATE_FALLBACK
    {"INAPPROPRIATE_FALLBACK", ERR_LIB_SSL, SSL_R_INAPPROPRIATE_FALLBACK},
  #else
    {"INAPPROPRIATE_FALLBACK", ERR_LIB_SSL, 373},
  #endif
  #ifdef SSL_R_INCONSISTENT_COMPRESSION
    {"INCONSISTENT_COMPRESSION", ERR_LIB_SSL, SSL_R_INCONSISTENT_COMPRESSION},
  #else
    {"INCONSISTENT_COMPRESSION", ERR_LIB_SSL, 340},
  #endif
  #ifdef SSL_R_INVALID_CHALLENGE_LENGTH
    {"INVALID_CHALLENGE_LENGTH", ERR_LIB_SSL, SSL_R_INVALID_CHALLENGE_LENGTH},
  #else
    {"INVALID_CHALLENGE_LENGTH", ERR_LIB_SSL, 158},
  #endif
  #ifdef SSL_R_INVALID_COMMAND
    {"INVALID_COMMAND", ERR_LIB_SSL, SSL_R_INVALID_COMMAND},
  #else
    {"INVALID_COMMAND", ERR_LIB_SSL, 280},
  #endif
  #ifdef SSL_R_INVALID_COMPRESSION_ALGORITHM
    {"INVALID_COMPRESSION_ALGORITHM", ERR_LIB_SSL, SSL_R_INVALID_COMPRESSION_ALGORITHM},
  #else
    {"INVALID_COMPRESSION_ALGORITHM", ERR_LIB_SSL, 341},
  #endif
  #ifdef SSL_R_INVALID_NULL_CMD_NAME
    {"INVALID_NULL_CMD_NAME", ERR_LIB_SSL, SSL_R_INVALID_NULL_CMD_NAME},
  #else
    {"INVALID_NULL_CMD_NAME", ERR_LIB_SSL, 385},
  #endif
  #ifdef SSL_R_INVALID_PURPOSE
    {"INVALID_PURPOSE", ERR_LIB_SSL, SSL_R_INVALID_PURPOSE},
  #else
    {"INVALID_PURPOSE", ERR_LIB_SSL, 278},
  #endif
  #ifdef SSL_R_INVALID_SERVERINFO_DATA
    {"INVALID_SERVERINFO_DATA", ERR_LIB_SSL, SSL_R_INVALID_SERVERINFO_DATA},
  #else
    {"INVALID_SERVERINFO_DATA", ERR_LIB_SSL, 388},
  #endif
  #ifdef SSL_R_INVALID_SRP_USERNAME
    {"INVALID_SRP_USERNAME", ERR_LIB_SSL, SSL_R_INVALID_SRP_USERNAME},
  #else
    {"INVALID_SRP_USERNAME", ERR_LIB_SSL, 357},
  #endif
  #ifdef SSL_R_INVALID_STATUS_RESPONSE
    {"INVALID_STATUS_RESPONSE", ERR_LIB_SSL, SSL_R_INVALID_STATUS_RESPONSE},
  #else
    {"INVALID_STATUS_RESPONSE", ERR_LIB_SSL, 328},
  #endif
  #ifdef SSL_R_INVALID_TICKET_KEYS_LENGTH
    {"INVALID_TICKET_KEYS_LENGTH", ERR_LIB_SSL, SSL_R_INVALID_TICKET_KEYS_LENGTH},
  #else
    {"INVALID_TICKET_KEYS_LENGTH", ERR_LIB_SSL, 325},
  #endif
  #ifdef SSL_R_INVALID_TRUST
    {"INVALID_TRUST", ERR_LIB_SSL, SSL_R_INVALID_TRUST},
  #else
    {"INVALID_TRUST", ERR_LIB_SSL, 279},
  #endif
  #ifdef SSL_R_KEY_ARG_TOO_LONG
    {"KEY_ARG_TOO_LONG", ERR_LIB_SSL, SSL_R_KEY_ARG_TOO_LONG},
  #else
    {"KEY_ARG_TOO_LONG", ERR_LIB_SSL, 284},
  #endif
  #ifdef SSL_R_KRB5
    {"KRB5", ERR_LIB_SSL, SSL_R_KRB5},
  #else
    {"KRB5", ERR_LIB_SSL, 285},
  #endif
  #ifdef SSL_R_KRB5_C_CC_PRINC
    {"KRB5_C_CC_PRINC", ERR_LIB_SSL, SSL_R_KRB5_C_CC_PRINC},
  #else
    {"KRB5_C_CC_PRINC", ERR_LIB_SSL, 286},
  #endif
  #ifdef SSL_R_KRB5_C_GET_CRED
    {"KRB5_C_GET_CRED", ERR_LIB_SSL, SSL_R_KRB5_C_GET_CRED},
  #else
    {"KRB5_C_GET_CRED", ERR_LIB_SSL, 287},
  #endif
  #ifdef SSL_R_KRB5_C_INIT
    {"KRB5_C_INIT", ERR_LIB_SSL, SSL_R_KRB5_C_INIT},
  #else
    {"KRB5_C_INIT", ERR_LIB_SSL, 288},
  #endif
  #ifdef SSL_R_KRB5_C_MK_REQ
    {"KRB5_C_MK_REQ", ERR_LIB_SSL, SSL_R_KRB5_C_MK_REQ},
  #else
    {"KRB5_C_MK_REQ", ERR_LIB_SSL, 289},
  #endif
  #ifdef SSL_R_KRB5_S_BAD_TICKET
    {"KRB5_S_BAD_TICKET", ERR_LIB_SSL, SSL_R_KRB5_S_BAD_TICKET},
  #else
    {"KRB5_S_BAD_TICKET", ERR_LIB_SSL, 290},
  #endif
  #ifdef SSL_R_KRB5_S_INIT
    {"KRB5_S_INIT", ERR_LIB_SSL, SSL_R_KRB5_S_INIT},
  #else
    {"KRB5_S_INIT", ERR_LIB_SSL, 291},
  #endif
  #ifdef SSL_R_KRB5_S_RD_REQ
    {"KRB5_S_RD_REQ", ERR_LIB_SSL, SSL_R_KRB5_S_RD_REQ},
  #else
    {"KRB5_S_RD_REQ", ERR_LIB_SSL, 292},
  #endif
  #ifdef SSL_R_KRB5_S_TKT_EXPIRED
    {"KRB5_S_TKT_EXPIRED", ERR_LIB_SSL, SSL_R_KRB5_S_TKT_EXPIRED},
  #else
    {"KRB5_S_TKT_EXPIRED", ERR_LIB_SSL, 293},
  #endif
  #ifdef SSL_R_KRB5_S_TKT_NYV
    {"KRB5_S_TKT_NYV", ERR_LIB_SSL, SSL_R_KRB5_S_TKT_NYV},
  #else
    {"KRB5_S_TKT_NYV", ERR_LIB_SSL, 294},
  #endif
  #ifdef SSL_R_KRB5_S_TKT_SKEW
    {"KRB5_S_TKT_SKEW", ERR_LIB_SSL, SSL_R_KRB5_S_TKT_SKEW},
  #else
    {"KRB5_S_TKT_SKEW", ERR_LIB_SSL, 295},
  #endif
  #ifdef SSL_R_LENGTH_MISMATCH
    {"LENGTH_MISMATCH", ERR_LIB_SSL, SSL_R_LENGTH_MISMATCH},
  #else
    {"LENGTH_MISMATCH", ERR_LIB_SSL, 159},
  #endif
  #ifdef SSL_R_LENGTH_TOO_SHORT
    {"LENGTH_TOO_SHORT", ERR_LIB_SSL, SSL_R_LENGTH_TOO_SHORT},
  #else
    {"LENGTH_TOO_SHORT", ERR_LIB_SSL, 160},
  #endif
  #ifdef SSL_R_LIBRARY_BUG
    {"LIBRARY_BUG", ERR_LIB_SSL, SSL_R_LIBRARY_BUG},
  #else
    {"LIBRARY_BUG", ERR_LIB_SSL, 274},
  #endif
  #ifdef SSL_R_LIBRARY_HAS_NO_CIPHERS
    {"LIBRARY_HAS_NO_CIPHERS", ERR_LIB_SSL, SSL_R_LIBRARY_HAS_NO_CIPHERS},
  #else
    {"LIBRARY_HAS_NO_CIPHERS", ERR_LIB_SSL, 161},
  #endif
  #ifdef SSL_R_MESSAGE_TOO_LONG
    {"MESSAGE_TOO_LONG", ERR_LIB_SSL, SSL_R_MESSAGE_TOO_LONG},
  #else
    {"MESSAGE_TOO_LONG", ERR_LIB_SSL, 296},
  #endif
  #ifdef SSL_R_MISSING_DH_DSA_CERT
    {"MISSING_DH_DSA_CERT", ERR_LIB_SSL, SSL_R_MISSING_DH_DSA_CERT},
  #else
    {"MISSING_DH_DSA_CERT", ERR_LIB_SSL, 162},
  #endif
  #ifdef SSL_R_MISSING_DH_KEY
    {"MISSING_DH_KEY", ERR_LIB_SSL, SSL_R_MISSING_DH_KEY},
  #else
    {"MISSING_DH_KEY", ERR_LIB_SSL, 163},
  #endif
  #ifdef SSL_R_MISSING_DH_RSA_CERT
    {"MISSING_DH_RSA_CERT", ERR_LIB_SSL, SSL_R_MISSING_DH_RSA_CERT},
  #else
    {"MISSING_DH_RSA_CERT", ERR_LIB_SSL, 164},
  #endif
  #ifdef SSL_R_MISSING_DSA_SIGNING_CERT
    {"MISSING_DSA_SIGNING_CERT", ERR_LIB_SSL, SSL_R_MISSING_DSA_SIGNING_CERT},
  #else
    {"MISSING_DSA_SIGNING_CERT", ERR_LIB_SSL, 165},
  #endif
  #ifdef SSL_R_MISSING_ECDH_CERT
    {"MISSING_ECDH_CERT", ERR_LIB_SSL, SSL_R_MISSING_ECDH_CERT},
  #else
    {"MISSING_ECDH_CERT", ERR_LIB_SSL, 382},
  #endif
  #ifdef SSL_R_MISSING_ECDSA_SIGNING_CERT
    {"MISSING_ECDSA_SIGNING_CERT", ERR_LIB_SSL, SSL_R_MISSING_ECDSA_SIGNING_CERT},
  #else
    {"MISSING_ECDSA_SIGNING_CERT", ERR_LIB_SSL, 381},
  #endif
  #ifdef SSL_R_MISSING_EXPORT_TMP_DH_KEY
    {"MISSING_EXPORT_TMP_DH_KEY", ERR_LIB_SSL, SSL_R_MISSING_EXPORT_TMP_DH_KEY},
  #else
    {"MISSING_EXPORT_TMP_DH_KEY", ERR_LIB_SSL, 166},
  #endif
  #ifdef SSL_R_MISSING_EXPORT_TMP_RSA_KEY
    {"MISSING_EXPORT_TMP_RSA_KEY", ERR_LIB_SSL, SSL_R_MISSING_EXPORT_TMP_RSA_KEY},
  #else
    {"MISSING_EXPORT_TMP_RSA_KEY", ERR_LIB_SSL, 167},
  #endif
  #ifdef SSL_R_MISSING_RSA_CERTIFICATE
    {"MISSING_RSA_CERTIFICATE", ERR_LIB_SSL, SSL_R_MISSING_RSA_CERTIFICATE},
  #else
    {"MISSING_RSA_CERTIFICATE", ERR_LIB_SSL, 168},
  #endif
  #ifdef SSL_R_MISSING_RSA_ENCRYPTING_CERT
    {"MISSING_RSA_ENCRYPTING_CERT", ERR_LIB_SSL, SSL_R_MISSING_RSA_ENCRYPTING_CERT},
  #else
    {"MISSING_RSA_ENCRYPTING_CERT", ERR_LIB_SSL, 169},
  #endif
  #ifdef SSL_R_MISSING_RSA_SIGNING_CERT
    {"MISSING_RSA_SIGNING_CERT", ERR_LIB_SSL, SSL_R_MISSING_RSA_SIGNING_CERT},
  #else
    {"MISSING_RSA_SIGNING_CERT", ERR_LIB_SSL, 170},
  #endif
  #ifdef SSL_R_MISSING_SRP_PARAM
    {"MISSING_SRP_PARAM", ERR_LIB_SSL, SSL_R_MISSING_SRP_PARAM},
  #else
    {"MISSING_SRP_PARAM", ERR_LIB_SSL, 358},
  #endif
  #ifdef SSL_R_MISSING_TMP_DH_KEY
    {"MISSING_TMP_DH_KEY", ERR_LIB_SSL, SSL_R_MISSING_TMP_DH_KEY},
  #else
    {"MISSING_TMP_DH_KEY", ERR_LIB_SSL, 171},
  #endif
  #ifdef SSL_R_MISSING_TMP_ECDH_KEY
    {"MISSING_TMP_ECDH_KEY", ERR_LIB_SSL, SSL_R_MISSING_TMP_ECDH_KEY},
  #else
    {"MISSING_TMP_ECDH_KEY", ERR_LIB_SSL, 311},
  #endif
  #ifdef SSL_R_MISSING_TMP_RSA_KEY
    {"MISSING_TMP_RSA_KEY", ERR_LIB_SSL, SSL_R_MISSING_TMP_RSA_KEY},
  #else
    {"MISSING_TMP_RSA_KEY", ERR_LIB_SSL, 172},
  #endif
  #ifdef SSL_R_MISSING_TMP_RSA_PKEY
    {"MISSING_TMP_RSA_PKEY", ERR_LIB_SSL, SSL_R_MISSING_TMP_RSA_PKEY},
  #else
    {"MISSING_TMP_RSA_PKEY", ERR_LIB_SSL, 173},
  #endif
  #ifdef SSL_R_MISSING_VERIFY_MESSAGE
    {"MISSING_VERIFY_MESSAGE", ERR_LIB_SSL, SSL_R_MISSING_VERIFY_MESSAGE},
  #else
    {"MISSING_VERIFY_MESSAGE", ERR_LIB_SSL, 174},
  #endif
  #ifdef SSL_R_MULTIPLE_SGC_RESTARTS
    {"MULTIPLE_SGC_RESTARTS", ERR_LIB_SSL, SSL_R_MULTIPLE_SGC_RESTARTS},
  #else
    {"MULTIPLE_SGC_RESTARTS", ERR_LIB_SSL, 346},
  #endif
  #ifdef SSL_R_NON_SSLV2_INITIAL_PACKET
    {"NON_SSLV2_INITIAL_PACKET", ERR_LIB_SSL, SSL_R_NON_SSLV2_INITIAL_PACKET},
  #else
    {"NON_SSLV2_INITIAL_PACKET", ERR_LIB_SSL, 175},
  #endif
  #ifdef SSL_R_NO_CERTIFICATES_RETURNED
    {"NO_CERTIFICATES_RETURNED", ERR_LIB_SSL, SSL_R_NO_CERTIFICATES_RETURNED},
  #else
    {"NO_CERTIFICATES_RETURNED", ERR_LIB_SSL, 176},
  #endif
  #ifdef SSL_R_NO_CERTIFICATE_ASSIGNED
    {"NO_CERTIFICATE_ASSIGNED", ERR_LIB_SSL, SSL_R_NO_CERTIFICATE_ASSIGNED},
  #else
    {"NO_CERTIFICATE_ASSIGNED", ERR_LIB_SSL, 177},
  #endif
  #ifdef SSL_R_NO_CERTIFICATE_RETURNED
    {"NO_CERTIFICATE_RETURNED", ERR_LIB_SSL, SSL_R_NO_CERTIFICATE_RETURNED},
  #else
    {"NO_CERTIFICATE_RETURNED", ERR_LIB_SSL, 178},
  #endif
  #ifdef SSL_R_NO_CERTIFICATE_SET
    {"NO_CERTIFICATE_SET", ERR_LIB_SSL, SSL_R_NO_CERTIFICATE_SET},
  #else
    {"NO_CERTIFICATE_SET", ERR_LIB_SSL, 179},
  #endif
  #ifdef SSL_R_NO_CERTIFICATE_SPECIFIED
    {"NO_CERTIFICATE_SPECIFIED", ERR_LIB_SSL, SSL_R_NO_CERTIFICATE_SPECIFIED},
  #else
    {"NO_CERTIFICATE_SPECIFIED", ERR_LIB_SSL, 180},
  #endif
  #ifdef SSL_R_NO_CIPHERS_AVAILABLE
    {"NO_CIPHERS_AVAILABLE", ERR_LIB_SSL, SSL_R_NO_CIPHERS_AVAILABLE},
  #else
    {"NO_CIPHERS_AVAILABLE", ERR_LIB_SSL, 181},
  #endif
  #ifdef SSL_R_NO_CIPHERS_PASSED
    {"NO_CIPHERS_PASSED", ERR_LIB_SSL, SSL_R_NO_CIPHERS_PASSED},
  #else
    {"NO_CIPHERS_PASSED", ERR_LIB_SSL, 182},
  #endif
  #ifdef SSL_R_NO_CIPHERS_SPECIFIED
    {"NO_CIPHERS_SPECIFIED", ERR_LIB_SSL, SSL_R_NO_CIPHERS_SPECIFIED},
  #else
    {"NO_CIPHERS_SPECIFIED", ERR_LIB_SSL, 183},
  #endif
  #ifdef SSL_R_NO_CIPHER_LIST
    {"NO_CIPHER_LIST", ERR_LIB_SSL, SSL_R_NO_CIPHER_LIST},
  #else
    {"NO_CIPHER_LIST", ERR_LIB_SSL, 184},
  #endif
  #ifdef SSL_R_NO_CIPHER_MATCH
    {"NO_CIPHER_MATCH", ERR_LIB_SSL, SSL_R_NO_CIPHER_MATCH},
  #else
    {"NO_CIPHER_MATCH", ERR_LIB_SSL, 185},
  #endif
  #ifdef SSL_R_NO_CLIENT_CERT_METHOD
    {"NO_CLIENT_CERT_METHOD", ERR_LIB_SSL, SSL_R_NO_CLIENT_CERT_METHOD},
  #else
    {"NO_CLIENT_CERT_METHOD", ERR_LIB_SSL, 331},
  #endif
  #ifdef SSL_R_NO_CLIENT_CERT_RECEIVED
    {"NO_CLIENT_CERT_RECEIVED", ERR_LIB_SSL, SSL_R_NO_CLIENT_CERT_RECEIVED},
  #else
    {"NO_CLIENT_CERT_RECEIVED", ERR_LIB_SSL, 186},
  #endif
  #ifdef SSL_R_NO_COMPRESSION_SPECIFIED
    {"NO_COMPRESSION_SPECIFIED", ERR_LIB_SSL, SSL_R_NO_COMPRESSION_SPECIFIED},
  #else
    {"NO_COMPRESSION_SPECIFIED", ERR_LIB_SSL, 187},
  #endif
  #ifdef SSL_R_NO_GOST_CERTIFICATE_SENT_BY_PEER
    {"NO_GOST_CERTIFICATE_SENT_BY_PEER", ERR_LIB_SSL, SSL_R_NO_GOST_CERTIFICATE_SENT_BY_PEER},
  #else
    {"NO_GOST_CERTIFICATE_SENT_BY_PEER", ERR_LIB_SSL, 330},
  #endif
  #ifdef SSL_R_NO_METHOD_SPECIFIED
    {"NO_METHOD_SPECIFIED", ERR_LIB_SSL, SSL_R_NO_METHOD_SPECIFIED},
  #else
    {"NO_METHOD_SPECIFIED", ERR_LIB_SSL, 188},
  #endif
  #ifdef SSL_R_NO_PEM_EXTENSIONS
    {"NO_PEM_EXTENSIONS", ERR_LIB_SSL, SSL_R_NO_PEM_EXTENSIONS},
  #else
    {"NO_PEM_EXTENSIONS", ERR_LIB_SSL, 389},
  #endif
  #ifdef SSL_R_NO_PRIVATEKEY
    {"NO_PRIVATEKEY", ERR_LIB_SSL, SSL_R_NO_PRIVATEKEY},
  #else
    {"NO_PRIVATEKEY", ERR_LIB_SSL, 189},
  #endif
  #ifdef SSL_R_NO_PRIVATE_KEY_ASSIGNED
    {"NO_PRIVATE_KEY_ASSIGNED", ERR_LIB_SSL, SSL_R_NO_PRIVATE_KEY_ASSIGNED},
  #else
    {"NO_PRIVATE_KEY_ASSIGNED", ERR_LIB_SSL, 190},
  #endif
  #ifdef SSL_R_NO_PROTOCOLS_AVAILABLE
    {"NO_PROTOCOLS_AVAILABLE", ERR_LIB_SSL, SSL_R_NO_PROTOCOLS_AVAILABLE},
  #else
    {"NO_PROTOCOLS_AVAILABLE", ERR_LIB_SSL, 191},
  #endif
  #ifdef SSL_R_NO_PUBLICKEY
    {"NO_PUBLICKEY", ERR_LIB_SSL, SSL_R_NO_PUBLICKEY},
  #else
    {"NO_PUBLICKEY", ERR_LIB_SSL, 192},
  #endif
  #ifdef SSL_R_NO_RENEGOTIATION
    {"NO_RENEGOTIATION", ERR_LIB_SSL, SSL_R_NO_RENEGOTIATION},
  #else
    {"NO_RENEGOTIATION", ERR_LIB_SSL, 339},
  #endif
  #ifdef SSL_R_NO_REQUIRED_DIGEST
    {"NO_REQUIRED_DIGEST", ERR_LIB_SSL, SSL_R_NO_REQUIRED_DIGEST},
  #else
    {"NO_REQUIRED_DIGEST", ERR_LIB_SSL, 324},
  #endif
  #ifdef SSL_R_NO_SHARED_CIPHER
    {"NO_SHARED_CIPHER", ERR_LIB_SSL, SSL_R_NO_SHARED_CIPHER},
  #else
    {"NO_SHARED_CIPHER", ERR_LIB_SSL, 193},
  #endif
  #ifdef SSL_R_NO_SHARED_SIGATURE_ALGORITHMS
    {"NO_SHARED_SIGATURE_ALGORITHMS", ERR_LIB_SSL, SSL_R_NO_SHARED_SIGATURE_ALGORITHMS},
  #else
    {"NO_SHARED_SIGATURE_ALGORITHMS", ERR_LIB_SSL, 376},
  #endif
  #ifdef SSL_R_NO_SRTP_PROFILES
    {"NO_SRTP_PROFILES", ERR_LIB_SSL, SSL_R_NO_SRTP_PROFILES},
  #else
    {"NO_SRTP_PROFILES", ERR_LIB_SSL, 359},
  #endif
  #ifdef SSL_R_NO_VERIFY_CALLBACK
    {"NO_VERIFY_CALLBACK", ERR_LIB_SSL, SSL_R_NO_VERIFY_CALLBACK},
  #else
    {"NO_VERIFY_CALLBACK", ERR_LIB_SSL, 194},
  #endif
  #ifdef SSL_R_NULL_SSL_CTX
    {"NULL_SSL_CTX", ERR_LIB_SSL, SSL_R_NULL_SSL_CTX},
  #else
    {"NULL_SSL_CTX", ERR_LIB_SSL, 195},
  #endif
  #ifdef SSL_R_NULL_SSL_METHOD_PASSED
    {"NULL_SSL_METHOD_PASSED", ERR_LIB_SSL, SSL_R_NULL_SSL_METHOD_PASSED},
  #else
    {"NULL_SSL_METHOD_PASSED", ERR_LIB_SSL, 196},
  #endif
  #ifdef SSL_R_OLD_SESSION_CIPHER_NOT_RETURNED
    {"OLD_SESSION_CIPHER_NOT_RETURNED", ERR_LIB_SSL, SSL_R_OLD_SESSION_CIPHER_NOT_RETURNED},
  #else
    {"OLD_SESSION_CIPHER_NOT_RETURNED", ERR_LIB_SSL, 197},
  #endif
  #ifdef SSL_R_OLD_SESSION_COMPRESSION_ALGORITHM_NOT_RETURNED
    {"OLD_SESSION_COMPRESSION_ALGORITHM_NOT_RETURNED", ERR_LIB_SSL, SSL_R_OLD_SESSION_COMPRESSION_ALGORITHM_NOT_RETURNED},
  #else
    {"OLD_SESSION_COMPRESSION_ALGORITHM_NOT_RETURNED", ERR_LIB_SSL, 344},
  #endif
  #ifdef SSL_R_ONLY_DTLS_1_2_ALLOWED_IN_SUITEB_MODE
    {"ONLY_DTLS_1_2_ALLOWED_IN_SUITEB_MODE", ERR_LIB_SSL, SSL_R_ONLY_DTLS_1_2_ALLOWED_IN_SUITEB_MODE},
  #else
    {"ONLY_DTLS_1_2_ALLOWED_IN_SUITEB_MODE", ERR_LIB_SSL, 387},
  #endif
  #ifdef SSL_R_ONLY_TLS_1_2_ALLOWED_IN_SUITEB_MODE
    {"ONLY_TLS_1_2_ALLOWED_IN_SUITEB_MODE", ERR_LIB_SSL, SSL_R_ONLY_TLS_1_2_ALLOWED_IN_SUITEB_MODE},
  #else
    {"ONLY_TLS_1_2_ALLOWED_IN_SUITEB_MODE", ERR_LIB_SSL, 379},
  #endif
  #ifdef SSL_R_ONLY_TLS_ALLOWED_IN_FIPS_MODE
    {"ONLY_TLS_ALLOWED_IN_FIPS_MODE", ERR_LIB_SSL, SSL_R_ONLY_TLS_ALLOWED_IN_FIPS_MODE},
  #else
    {"ONLY_TLS_ALLOWED_IN_FIPS_MODE", ERR_LIB_SSL, 297},
  #endif
  #ifdef SSL_R_OPAQUE_PRF_INPUT_TOO_LONG
    {"OPAQUE_PRF_INPUT_TOO_LONG", ERR_LIB_SSL, SSL_R_OPAQUE_PRF_INPUT_TOO_LONG},
  #else
    {"OPAQUE_PRF_INPUT_TOO_LONG", ERR_LIB_SSL, 327},
  #endif
  #ifdef SSL_R_PACKET_LENGTH_TOO_LONG
    {"PACKET_LENGTH_TOO_LONG", ERR_LIB_SSL, SSL_R_PACKET_LENGTH_TOO_LONG},
  #else
    {"PACKET_LENGTH_TOO_LONG", ERR_LIB_SSL, 198},
  #endif
  #ifdef SSL_R_PARSE_TLSEXT
    {"PARSE_TLSEXT", ERR_LIB_SSL, SSL_R_PARSE_TLSEXT},
  #else
    {"PARSE_TLSEXT", ERR_LIB_SSL, 227},
  #endif
  #ifdef SSL_R_PATH_TOO_LONG
    {"PATH_TOO_LONG", ERR_LIB_SSL, SSL_R_PATH_TOO_LONG},
  #else
    {"PATH_TOO_LONG", ERR_LIB_SSL, 270},
  #endif
  #ifdef SSL_R_PEER_DID_NOT_RETURN_A_CERTIFICATE
    {"PEER_DID_NOT_RETURN_A_CERTIFICATE", ERR_LIB_SSL, SSL_R_PEER_DID_NOT_RETURN_A_CERTIFICATE},
  #else
    {"PEER_DID_NOT_RETURN_A_CERTIFICATE", ERR_LIB_SSL, 199},
  #endif
  #ifdef SSL_R_PEER_ERROR
    {"PEER_ERROR", ERR_LIB_SSL, SSL_R_PEER_ERROR},
  #else
    {"PEER_ERROR", ERR_LIB_SSL, 200},
  #endif
  #ifdef SSL_R_PEER_ERROR_CERTIFICATE
    {"PEER_ERROR_CERTIFICATE", ERR_LIB_SSL, SSL_R_PEER_ERROR_CERTIFICATE},
  #else
    {"PEER_ERROR_CERTIFICATE", ERR_LIB_SSL, 201},
  #endif
  #ifdef SSL_R_PEER_ERROR_NO_CERTIFICATE
    {"PEER_ERROR_NO_CERTIFICATE", ERR_LIB_SSL, SSL_R_PEER_ERROR_NO_CERTIFICATE},
  #else
    {"PEER_ERROR_NO_CERTIFICATE", ERR_LIB_SSL, 202},
  #endif
  #ifdef SSL_R_PEER_ERROR_NO_CIPHER
    {"PEER_ERROR_NO_CIPHER", ERR_LIB_SSL, SSL_R_PEER_ERROR_NO_CIPHER},
  #else
    {"PEER_ERROR_NO_CIPHER", ERR_LIB_SSL, 203},
  #endif
  #ifdef SSL_R_PEER_ERROR_UNSUPPORTED_CERTIFICATE_TYPE
    {"PEER_ERROR_UNSUPPORTED_CERTIFICATE_TYPE", ERR_LIB_SSL, SSL_R_PEER_ERROR_UNSUPPORTED_CERTIFICATE_TYPE},
  #else
    {"PEER_ERROR_UNSUPPORTED_CERTIFICATE_TYPE", ERR_LIB_SSL, 204},
  #endif
  #ifdef SSL_R_PEM_NAME_BAD_PREFIX
    {"PEM_NAME_BAD_PREFIX", ERR_LIB_SSL, SSL_R_PEM_NAME_BAD_PREFIX},
  #else
    {"PEM_NAME_BAD_PREFIX", ERR_LIB_SSL, 391},
  #endif
  #ifdef SSL_R_PEM_NAME_TOO_SHORT
    {"PEM_NAME_TOO_SHORT", ERR_LIB_SSL, SSL_R_PEM_NAME_TOO_SHORT},
  #else
    {"PEM_NAME_TOO_SHORT", ERR_LIB_SSL, 392},
  #endif
  #ifdef SSL_R_PRE_MAC_LENGTH_TOO_LONG
    {"PRE_MAC_LENGTH_TOO_LONG", ERR_LIB_SSL, SSL_R_PRE_MAC_LENGTH_TOO_LONG},
  #else
    {"PRE_MAC_LENGTH_TOO_LONG", ERR_LIB_SSL, 205},
  #endif
  #ifdef SSL_R_PROBLEMS_MAPPING_CIPHER_FUNCTIONS
    {"PROBLEMS_MAPPING_CIPHER_FUNCTIONS", ERR_LIB_SSL, SSL_R_PROBLEMS_MAPPING_CIPHER_FUNCTIONS},
  #else
    {"PROBLEMS_MAPPING_CIPHER_FUNCTIONS", ERR_LIB_SSL, 206},
  #endif
  #ifdef SSL_R_PROTOCOL_IS_SHUTDOWN
    {"PROTOCOL_IS_SHUTDOWN", ERR_LIB_SSL, SSL_R_PROTOCOL_IS_SHUTDOWN},
  #else
    {"PROTOCOL_IS_SHUTDOWN", ERR_LIB_SSL, 207},
  #endif
  #ifdef SSL_R_PSK_IDENTITY_NOT_FOUND
    {"PSK_IDENTITY_NOT_FOUND", ERR_LIB_SSL, SSL_R_PSK_IDENTITY_NOT_FOUND},
  #else
    {"PSK_IDENTITY_NOT_FOUND", ERR_LIB_SSL, 223},
  #endif
  #ifdef SSL_R_PSK_NO_CLIENT_CB
    {"PSK_NO_CLIENT_CB", ERR_LIB_SSL, SSL_R_PSK_NO_CLIENT_CB},
  #else
    {"PSK_NO_CLIENT_CB", ERR_LIB_SSL, 224},
  #endif
  #ifdef SSL_R_PSK_NO_SERVER_CB
    {"PSK_NO_SERVER_CB", ERR_LIB_SSL, SSL_R_PSK_NO_SERVER_CB},
  #else
    {"PSK_NO_SERVER_CB", ERR_LIB_SSL, 225},
  #endif
  #ifdef SSL_R_PUBLIC_KEY_ENCRYPT_ERROR
    {"PUBLIC_KEY_ENCRYPT_ERROR", ERR_LIB_SSL, SSL_R_PUBLIC_KEY_ENCRYPT_ERROR},
  #else
    {"PUBLIC_KEY_ENCRYPT_ERROR", ERR_LIB_SSL, 208},
  #endif
  #ifdef SSL_R_PUBLIC_KEY_IS_NOT_RSA
    {"PUBLIC_KEY_IS_NOT_RSA", ERR_LIB_SSL, SSL_R_PUBLIC_KEY_IS_NOT_RSA},
  #else
    {"PUBLIC_KEY_IS_NOT_RSA", ERR_LIB_SSL, 209},
  #endif
  #ifdef SSL_R_PUBLIC_KEY_NOT_RSA
    {"PUBLIC_KEY_NOT_RSA", ERR_LIB_SSL, SSL_R_PUBLIC_KEY_NOT_RSA},
  #else
    {"PUBLIC_KEY_NOT_RSA", ERR_LIB_SSL, 210},
  #endif
  #ifdef SSL_R_READ_BIO_NOT_SET
    {"READ_BIO_NOT_SET", ERR_LIB_SSL, SSL_R_READ_BIO_NOT_SET},
  #else
    {"READ_BIO_NOT_SET", ERR_LIB_SSL, 211},
  #endif
  #ifdef SSL_R_READ_TIMEOUT_EXPIRED
    {"READ_TIMEOUT_EXPIRED", ERR_LIB_SSL, SSL_R_READ_TIMEOUT_EXPIRED},
  #else
    {"READ_TIMEOUT_EXPIRED", ERR_LIB_SSL, 312},
  #endif
  #ifdef SSL_R_READ_WRONG_PACKET_TYPE
    {"READ_WRONG_PACKET_TYPE", ERR_LIB_SSL, SSL_R_READ_WRONG_PACKET_TYPE},
  #else
    {"READ_WRONG_PACKET_TYPE", ERR_LIB_SSL, 212},
  #endif
  #ifdef SSL_R_RECORD_LENGTH_MISMATCH
    {"RECORD_LENGTH_MISMATCH", ERR_LIB_SSL, SSL_R_RECORD_LENGTH_MISMATCH},
  #else
    {"RECORD_LENGTH_MISMATCH", ERR_LIB_SSL, 213},
  #endif
  #ifdef SSL_R_RECORD_TOO_LARGE
    {"RECORD_TOO_LARGE", ERR_LIB_SSL, SSL_R_RECORD_TOO_LARGE},
  #else
    {"RECORD_TOO_LARGE", ERR_LIB_SSL, 214},
  #endif
  #ifdef SSL_R_RECORD_TOO_SMALL
    {"RECORD_TOO_SMALL", ERR_LIB_SSL, SSL_R_RECORD_TOO_SMALL},
  #else
    {"RECORD_TOO_SMALL", ERR_LIB_SSL, 298},
  #endif
  #ifdef SSL_R_RENEGOTIATE_EXT_TOO_LONG
    {"RENEGOTIATE_EXT_TOO_LONG", ERR_LIB_SSL, SSL_R_RENEGOTIATE_EXT_TOO_LONG},
  #else
    {"RENEGOTIATE_EXT_TOO_LONG", ERR_LIB_SSL, 335},
  #endif
  #ifdef SSL_R_RENEGOTIATION_ENCODING_ERR
    {"RENEGOTIATION_ENCODING_ERR", ERR_LIB_SSL, SSL_R_RENEGOTIATION_ENCODING_ERR},
  #else
    {"RENEGOTIATION_ENCODING_ERR", ERR_LIB_SSL, 336},
  #endif
  #ifdef SSL_R_RENEGOTIATION_MISMATCH
    {"RENEGOTIATION_MISMATCH", ERR_LIB_SSL, SSL_R_RENEGOTIATION_MISMATCH},
  #else
    {"RENEGOTIATION_MISMATCH", ERR_LIB_SSL, 337},
  #endif
  #ifdef SSL_R_REQUIRED_CIPHER_MISSING
    {"REQUIRED_CIPHER_MISSING", ERR_LIB_SSL, SSL_R_REQUIRED_CIPHER_MISSING},
  #else
    {"REQUIRED_CIPHER_MISSING", ERR_LIB_SSL, 215},
  #endif
  #ifdef SSL_R_REQUIRED_COMPRESSSION_ALGORITHM_MISSING
    {"REQUIRED_COMPRESSSION_ALGORITHM_MISSING", ERR_LIB_SSL, SSL_R_REQUIRED_COMPRESSSION_ALGORITHM_MISSING},
  #else
    {"REQUIRED_COMPRESSSION_ALGORITHM_MISSING", ERR_LIB_SSL, 342},
  #endif
  #ifdef SSL_R_REUSE_CERT_LENGTH_NOT_ZERO
    {"REUSE_CERT_LENGTH_NOT_ZERO", ERR_LIB_SSL, SSL_R_REUSE_CERT_LENGTH_NOT_ZERO},
  #else
    {"REUSE_CERT_LENGTH_NOT_ZERO", ERR_LIB_SSL, 216},
  #endif
  #ifdef SSL_R_REUSE_CERT_TYPE_NOT_ZERO
    {"REUSE_CERT_TYPE_NOT_ZERO", ERR_LIB_SSL, SSL_R_REUSE_CERT_TYPE_NOT_ZERO},
  #else
    {"REUSE_CERT_TYPE_NOT_ZERO", ERR_LIB_SSL, 217},
  #endif
  #ifdef SSL_R_REUSE_CIPHER_LIST_NOT_ZERO
    {"REUSE_CIPHER_LIST_NOT_ZERO", ERR_LIB_SSL, SSL_R_REUSE_CIPHER_LIST_NOT_ZERO},
  #else
    {"REUSE_CIPHER_LIST_NOT_ZERO", ERR_LIB_SSL, 218},
  #endif
  #ifdef SSL_R_SCSV_RECEIVED_WHEN_RENEGOTIATING
    {"SCSV_RECEIVED_WHEN_RENEGOTIATING", ERR_LIB_SSL, SSL_R_SCSV_RECEIVED_WHEN_RENEGOTIATING},
  #else
    {"SCSV_RECEIVED_WHEN_RENEGOTIATING", ERR_LIB_SSL, 345},
  #endif
  #ifdef SSL_R_SERVERHELLO_TLSEXT
    {"SERVERHELLO_TLSEXT", ERR_LIB_SSL, SSL_R_SERVERHELLO_TLSEXT},
  #else
    {"SERVERHELLO_TLSEXT", ERR_LIB_SSL, 275},
  #endif
  #ifdef SSL_R_SESSION_ID_CONTEXT_UNINITIALIZED
    {"SESSION_ID_CONTEXT_UNINITIALIZED", ERR_LIB_SSL, SSL_R_SESSION_ID_CONTEXT_UNINITIALIZED},
  #else
    {"SESSION_ID_CONTEXT_UNINITIALIZED", ERR_LIB_SSL, 277},
  #endif
  #ifdef SSL_R_SHORT_READ
    {"SHORT_READ", ERR_LIB_SSL, SSL_R_SHORT_READ},
  #else
    {"SHORT_READ", ERR_LIB_SSL, 219},
  #endif
  #ifdef SSL_R_SIGNATURE_ALGORITHMS_ERROR
    {"SIGNATURE_ALGORITHMS_ERROR", ERR_LIB_SSL, SSL_R_SIGNATURE_ALGORITHMS_ERROR},
  #else
    {"SIGNATURE_ALGORITHMS_ERROR", ERR_LIB_SSL, 360},
  #endif
  #ifdef SSL_R_SIGNATURE_FOR_NON_SIGNING_CERTIFICATE
    {"SIGNATURE_FOR_NON_SIGNING_CERTIFICATE", ERR_LIB_SSL, SSL_R_SIGNATURE_FOR_NON_SIGNING_CERTIFICATE},
  #else
    {"SIGNATURE_FOR_NON_SIGNING_CERTIFICATE", ERR_LIB_SSL, 220},
  #endif
  #ifdef SSL_R_SRP_A_CALC
    {"SRP_A_CALC", ERR_LIB_SSL, SSL_R_SRP_A_CALC},
  #else
    {"SRP_A_CALC", ERR_LIB_SSL, 361},
  #endif
  #ifdef SSL_R_SRTP_COULD_NOT_ALLOCATE_PROFILES
    {"SRTP_COULD_NOT_ALLOCATE_PROFILES", ERR_LIB_SSL, SSL_R_SRTP_COULD_NOT_ALLOCATE_PROFILES},
  #else
    {"SRTP_COULD_NOT_ALLOCATE_PROFILES", ERR_LIB_SSL, 362},
  #endif
  #ifdef SSL_R_SRTP_PROTECTION_PROFILE_LIST_TOO_LONG
    {"SRTP_PROTECTION_PROFILE_LIST_TOO_LONG", ERR_LIB_SSL, SSL_R_SRTP_PROTECTION_PROFILE_LIST_TOO_LONG},
  #else
    {"SRTP_PROTECTION_PROFILE_LIST_TOO_LONG", ERR_LIB_SSL, 363},
  #endif
  #ifdef SSL_R_SRTP_UNKNOWN_PROTECTION_PROFILE
    {"SRTP_UNKNOWN_PROTECTION_PROFILE", ERR_LIB_SSL, SSL_R_SRTP_UNKNOWN_PROTECTION_PROFILE},
  #else
    {"SRTP_UNKNOWN_PROTECTION_PROFILE", ERR_LIB_SSL, 364},
  #endif
  #ifdef SSL_R_SSL23_DOING_SESSION_ID_REUSE
    {"SSL23_DOING_SESSION_ID_REUSE", ERR_LIB_SSL, SSL_R_SSL23_DOING_SESSION_ID_REUSE},
  #else
    {"SSL23_DOING_SESSION_ID_REUSE", ERR_LIB_SSL, 221},
  #endif
  #ifdef SSL_R_SSL2_CONNECTION_ID_TOO_LONG
    {"SSL2_CONNECTION_ID_TOO_LONG", ERR_LIB_SSL, SSL_R_SSL2_CONNECTION_ID_TOO_LONG},
  #else
    {"SSL2_CONNECTION_ID_TOO_LONG", ERR_LIB_SSL, 299},
  #endif
  #ifdef SSL_R_SSL3_EXT_INVALID_ECPOINTFORMAT
    {"SSL3_EXT_INVALID_ECPOINTFORMAT", ERR_LIB_SSL, SSL_R_SSL3_EXT_INVALID_ECPOINTFORMAT},
  #else
    {"SSL3_EXT_INVALID_ECPOINTFORMAT", ERR_LIB_SSL, 321},
  #endif
  #ifdef SSL_R_SSL3_EXT_INVALID_SERVERNAME
    {"SSL3_EXT_INVALID_SERVERNAME", ERR_LIB_SSL, SSL_R_SSL3_EXT_INVALID_SERVERNAME},
  #else
    {"SSL3_EXT_INVALID_SERVERNAME", ERR_LIB_SSL, 319},
  #endif
  #ifdef SSL_R_SSL3_EXT_INVALID_SERVERNAME_TYPE
    {"SSL3_EXT_INVALID_SERVERNAME_TYPE", ERR_LIB_SSL, SSL_R_SSL3_EXT_INVALID_SERVERNAME_TYPE},
  #else
    {"SSL3_EXT_INVALID_SERVERNAME_TYPE", ERR_LIB_SSL, 320},
  #endif
  #ifdef SSL_R_SSL3_SESSION_ID_TOO_LONG
    {"SSL3_SESSION_ID_TOO_LONG", ERR_LIB_SSL, SSL_R_SSL3_SESSION_ID_TOO_LONG},
  #else
    {"SSL3_SESSION_ID_TOO_LONG", ERR_LIB_SSL, 300},
  #endif
  #ifdef SSL_R_SSL3_SESSION_ID_TOO_SHORT
    {"SSL3_SESSION_ID_TOO_SHORT", ERR_LIB_SSL, SSL_R_SSL3_SESSION_ID_TOO_SHORT},
  #else
    {"SSL3_SESSION_ID_TOO_SHORT", ERR_LIB_SSL, 222},
  #endif
  #ifdef SSL_R_SSLV3_ALERT_BAD_CERTIFICATE
    {"SSLV3_ALERT_BAD_CERTIFICATE", ERR_LIB_SSL, SSL_R_SSLV3_ALERT_BAD_CERTIFICATE},
  #else
    {"SSLV3_ALERT_BAD_CERTIFICATE", ERR_LIB_SSL, 1042},
  #endif
  #ifdef SSL_R_SSLV3_ALERT_BAD_RECORD_MAC
    {"SSLV3_ALERT_BAD_RECORD_MAC", ERR_LIB_SSL, SSL_R_SSLV3_ALERT_BAD_RECORD_MAC},
  #else
    {"SSLV3_ALERT_BAD_RECORD_MAC", ERR_LIB_SSL, 1020},
  #endif
  #ifdef SSL_R_SSLV3_ALERT_CERTIFICATE_EXPIRED
    {"SSLV3_ALERT_CERTIFICATE_EXPIRED", ERR_LIB_SSL, SSL_R_SSLV3_ALERT_CERTIFICATE_EXPIRED},
  #else
    {"SSLV3_ALERT_CERTIFICATE_EXPIRED", ERR_LIB_SSL, 1045},
  #endif
  #ifdef SSL_R_SSLV3_ALERT_CERTIFICATE_REVOKED
    {"SSLV3_ALERT_CERTIFICATE_REVOKED", ERR_LIB_SSL, SSL_R_SSLV3_ALERT_CERTIFICATE_REVOKED},
  #else
    {"SSLV3_ALERT_CERTIFICATE_REVOKED", ERR_LIB_SSL, 1044},
  #endif
  #ifdef SSL_R_SSLV3_ALERT_CERTIFICATE_UNKNOWN
    {"SSLV3_ALERT_CERTIFICATE_UNKNOWN", ERR_LIB_SSL, SSL_R_SSLV3_ALERT_CERTIFICATE_UNKNOWN},
  #else
    {"SSLV3_ALERT_CERTIFICATE_UNKNOWN", ERR_LIB_SSL, 1046},
  #endif
  #ifdef SSL_R_SSLV3_ALERT_DECOMPRESSION_FAILURE
    {"SSLV3_ALERT_DECOMPRESSION_FAILURE", ERR_LIB_SSL, SSL_R_SSLV3_ALERT_DECOMPRESSION_FAILURE},
  #else
    {"SSLV3_ALERT_DECOMPRESSION_FAILURE", ERR_LIB_SSL, 1030},
  #endif
  #ifdef SSL_R_SSLV3_ALERT_HANDSHAKE_FAILURE
    {"SSLV3_ALERT_HANDSHAKE_FAILURE", ERR_LIB_SSL, SSL_R_SSLV3_ALERT_HANDSHAKE_FAILURE},
  #else
    {"SSLV3_ALERT_HANDSHAKE_FAILURE", ERR_LIB_SSL, 1040},
  #endif
  #ifdef SSL_R_SSLV3_ALERT_ILLEGAL_PARAMETER
    {"SSLV3_ALERT_ILLEGAL_PARAMETER", ERR_LIB_SSL, SSL_R_SSLV3_ALERT_ILLEGAL_PARAMETER},
  #else
    {"SSLV3_ALERT_ILLEGAL_PARAMETER", ERR_LIB_SSL, 1047},
  #endif
  #ifdef SSL_R_SSLV3_ALERT_NO_CERTIFICATE
    {"SSLV3_ALERT_NO_CERTIFICATE", ERR_LIB_SSL, SSL_R_SSLV3_ALERT_NO_CERTIFICATE},
  #else
    {"SSLV3_ALERT_NO_CERTIFICATE", ERR_LIB_SSL, 1041},
  #endif
  #ifdef SSL_R_SSLV3_ALERT_UNEXPECTED_MESSAGE
    {"SSLV3_ALERT_UNEXPECTED_MESSAGE", ERR_LIB_SSL, SSL_R_SSLV3_ALERT_UNEXPECTED_MESSAGE},
  #else
    {"SSLV3_ALERT_UNEXPECTED_MESSAGE", ERR_LIB_SSL, 1010},
  #endif
  #ifdef SSL_R_SSLV3_ALERT_UNSUPPORTED_CERTIFICATE
    {"SSLV3_ALERT_UNSUPPORTED_CERTIFICATE", ERR_LIB_SSL, SSL_R_SSLV3_ALERT_UNSUPPORTED_CERTIFICATE},
  #else
    {"SSLV3_ALERT_UNSUPPORTED_CERTIFICATE", ERR_LIB_SSL, 1043},
  #endif
  #ifdef SSL_R_SSL_CTX_HAS_NO_DEFAULT_SSL_VERSION
    {"SSL_CTX_HAS_NO_DEFAULT_SSL_VERSION", ERR_LIB_SSL, SSL_R_SSL_CTX_HAS_NO_DEFAULT_SSL_VERSION},
  #else
    {"SSL_CTX_HAS_NO_DEFAULT_SSL_VERSION", ERR_LIB_SSL, 228},
  #endif
  #ifdef SSL_R_SSL_HANDSHAKE_FAILURE
    {"SSL_HANDSHAKE_FAILURE", ERR_LIB_SSL, SSL_R_SSL_HANDSHAKE_FAILURE},
  #else
    {"SSL_HANDSHAKE_FAILURE", ERR_LIB_SSL, 229},
  #endif
  #ifdef SSL_R_SSL_LIBRARY_HAS_NO_CIPHERS
    {"SSL_LIBRARY_HAS_NO_CIPHERS", ERR_LIB_SSL, SSL_R_SSL_LIBRARY_HAS_NO_CIPHERS},
  #else
    {"SSL_LIBRARY_HAS_NO_CIPHERS", ERR_LIB_SSL, 230},
  #endif
  #ifdef SSL_R_SSL_NEGATIVE_LENGTH
    {"SSL_NEGATIVE_LENGTH", ERR_LIB_SSL, SSL_R_SSL_NEGATIVE_LENGTH},
  #else
    {"SSL_NEGATIVE_LENGTH", ERR_LIB_SSL, 372},
  #endif
  #ifdef SSL_R_SSL_SESSION_ID_CALLBACK_FAILED
    {"SSL_SESSION_ID_CALLBACK_FAILED", ERR_LIB_SSL, SSL_R_SSL_SESSION_ID_CALLBACK_FAILED},
  #else
    {"SSL_SESSION_ID_CALLBACK_FAILED", ERR_LIB_SSL, 301},
  #endif
  #ifdef SSL_R_SSL_SESSION_ID_CONFLICT
    {"SSL_SESSION_ID_CONFLICT", ERR_LIB_SSL, SSL_R_SSL_SESSION_ID_CONFLICT},
  #else
    {"SSL_SESSION_ID_CONFLICT", ERR_LIB_SSL, 302},
  #endif
  #ifdef SSL_R_SSL_SESSION_ID_CONTEXT_TOO_LONG
    {"SSL_SESSION_ID_CONTEXT_TOO_LONG", ERR_LIB_SSL, SSL_R_SSL_SESSION_ID_CONTEXT_TOO_LONG},
  #else
    {"SSL_SESSION_ID_CONTEXT_TOO_LONG", ERR_LIB_SSL, 273},
  #endif
  #ifdef SSL_R_SSL_SESSION_ID_HAS_BAD_LENGTH
    {"SSL_SESSION_ID_HAS_BAD_LENGTH", ERR_LIB_SSL, SSL_R_SSL_SESSION_ID_HAS_BAD_LENGTH},
  #else
    {"SSL_SESSION_ID_HAS_BAD_LENGTH", ERR_LIB_SSL, 303},
  #endif
  #ifdef SSL_R_SSL_SESSION_ID_IS_DIFFERENT
    {"SSL_SESSION_ID_IS_DIFFERENT", ERR_LIB_SSL, SSL_R_SSL_SESSION_ID_IS_DIFFERENT},
  #else
    {"SSL_SESSION_ID_IS_DIFFERENT", ERR_LIB_SSL, 231},
  #endif
  #ifdef SSL_R_TLSV1_ALERT_ACCESS_DENIED
    {"TLSV1_ALERT_ACCESS_DENIED", ERR_LIB_SSL, SSL_R_TLSV1_ALERT_ACCESS_DENIED},
  #else
    {"TLSV1_ALERT_ACCESS_DENIED", ERR_LIB_SSL, 1049},
  #endif
  #ifdef SSL_R_TLSV1_ALERT_DECODE_ERROR
    {"TLSV1_ALERT_DECODE_ERROR", ERR_LIB_SSL, SSL_R_TLSV1_ALERT_DECODE_ERROR},
  #else
    {"TLSV1_ALERT_DECODE_ERROR", ERR_LIB_SSL, 1050},
  #endif
  #ifdef SSL_R_TLSV1_ALERT_DECRYPTION_FAILED
    {"TLSV1_ALERT_DECRYPTION_FAILED", ERR_LIB_SSL, SSL_R_TLSV1_ALERT_DECRYPTION_FAILED},
  #else
    {"TLSV1_ALERT_DECRYPTION_FAILED", ERR_LIB_SSL, 1021},
  #endif
  #ifdef SSL_R_TLSV1_ALERT_DECRYPT_ERROR
    {"TLSV1_ALERT_DECRYPT_ERROR", ERR_LIB_SSL, SSL_R_TLSV1_ALERT_DECRYPT_ERROR},
  #else
    {"TLSV1_ALERT_DECRYPT_ERROR", ERR_LIB_SSL, 1051},
  #endif
  #ifdef SSL_R_TLSV1_ALERT_EXPORT_RESTRICTION
    {"TLSV1_ALERT_EXPORT_RESTRICTION", ERR_LIB_SSL, SSL_R_TLSV1_ALERT_EXPORT_RESTRICTION},
  #else
    {"TLSV1_ALERT_EXPORT_RESTRICTION", ERR_LIB_SSL, 1060},
  #endif
  #ifdef SSL_R_TLSV1_ALERT_INAPPROPRIATE_FALLBACK
    {"TLSV1_ALERT_INAPPROPRIATE_FALLBACK", ERR_LIB_SSL, SSL_R_TLSV1_ALERT_INAPPROPRIATE_FALLBACK},
  #else
    {"TLSV1_ALERT_INAPPROPRIATE_FALLBACK", ERR_LIB_SSL, 1086},
  #endif
  #ifdef SSL_R_TLSV1_ALERT_INSUFFICIENT_SECURITY
    {"TLSV1_ALERT_INSUFFICIENT_SECURITY", ERR_LIB_SSL, SSL_R_TLSV1_ALERT_INSUFFICIENT_SECURITY},
  #else
    {"TLSV1_ALERT_INSUFFICIENT_SECURITY", ERR_LIB_SSL, 1071},
  #endif
  #ifdef SSL_R_TLSV1_ALERT_INTERNAL_ERROR
    {"TLSV1_ALERT_INTERNAL_ERROR", ERR_LIB_SSL, SSL_R_TLSV1_ALERT_INTERNAL_ERROR},
  #else
    {"TLSV1_ALERT_INTERNAL_ERROR", ERR_LIB_SSL, 1080},
  #endif
  #ifdef SSL_R_TLSV1_ALERT_NO_RENEGOTIATION
    {"TLSV1_ALERT_NO_RENEGOTIATION", ERR_LIB_SSL, SSL_R_TLSV1_ALERT_NO_RENEGOTIATION},
  #else
    {"TLSV1_ALERT_NO_RENEGOTIATION", ERR_LIB_SSL, 1100},
  #endif
  #ifdef SSL_R_TLSV1_ALERT_PROTOCOL_VERSION
    {"TLSV1_ALERT_PROTOCOL_VERSION", ERR_LIB_SSL, SSL_R_TLSV1_ALERT_PROTOCOL_VERSION},
  #else
    {"TLSV1_ALERT_PROTOCOL_VERSION", ERR_LIB_SSL, 1070},
  #endif
  #ifdef SSL_R_TLSV1_ALERT_RECORD_OVERFLOW
    {"TLSV1_ALERT_RECORD_OVERFLOW", ERR_LIB_SSL, SSL_R_TLSV1_ALERT_RECORD_OVERFLOW},
  #else
    {"TLSV1_ALERT_RECORD_OVERFLOW", ERR_LIB_SSL, 1022},
  #endif
  #ifdef SSL_R_TLSV1_ALERT_UNKNOWN_CA
    {"TLSV1_ALERT_UNKNOWN_CA", ERR_LIB_SSL, SSL_R_TLSV1_ALERT_UNKNOWN_CA},
  #else
    {"TLSV1_ALERT_UNKNOWN_CA", ERR_LIB_SSL, 1048},
  #endif
  #ifdef SSL_R_TLSV1_ALERT_USER_CANCELLED
    {"TLSV1_ALERT_USER_CANCELLED", ERR_LIB_SSL, SSL_R_TLSV1_ALERT_USER_CANCELLED},
  #else
    {"TLSV1_ALERT_USER_CANCELLED", ERR_LIB_SSL, 1090},
  #endif
  #ifdef SSL_R_TLSV1_BAD_CERTIFICATE_HASH_VALUE
    {"TLSV1_BAD_CERTIFICATE_HASH_VALUE", ERR_LIB_SSL, SSL_R_TLSV1_BAD_CERTIFICATE_HASH_VALUE},
  #else
    {"TLSV1_BAD_CERTIFICATE_HASH_VALUE", ERR_LIB_SSL, 1114},
  #endif
  #ifdef SSL_R_TLSV1_BAD_CERTIFICATE_STATUS_RESPONSE
    {"TLSV1_BAD_CERTIFICATE_STATUS_RESPONSE", ERR_LIB_SSL, SSL_R_TLSV1_BAD_CERTIFICATE_STATUS_RESPONSE},
  #else
    {"TLSV1_BAD_CERTIFICATE_STATUS_RESPONSE", ERR_LIB_SSL, 1113},
  #endif
  #ifdef SSL_R_TLSV1_CERTIFICATE_UNOBTAINABLE
    {"TLSV1_CERTIFICATE_UNOBTAINABLE", ERR_LIB_SSL, SSL_R_TLSV1_CERTIFICATE_UNOBTAINABLE},
  #else
    {"TLSV1_CERTIFICATE_UNOBTAINABLE", ERR_LIB_SSL, 1111},
  #endif
  #ifdef SSL_R_TLSV1_UNRECOGNIZED_NAME
    {"TLSV1_UNRECOGNIZED_NAME", ERR_LIB_SSL, SSL_R_TLSV1_UNRECOGNIZED_NAME},
  #else
    {"TLSV1_UNRECOGNIZED_NAME", ERR_LIB_SSL, 1112},
  #endif
  #ifdef SSL_R_TLSV1_UNSUPPORTED_EXTENSION
    {"TLSV1_UNSUPPORTED_EXTENSION", ERR_LIB_SSL, SSL_R_TLSV1_UNSUPPORTED_EXTENSION},
  #else
    {"TLSV1_UNSUPPORTED_EXTENSION", ERR_LIB_SSL, 1110},
  #endif
  #ifdef SSL_R_TLS_CLIENT_CERT_REQ_WITH_ANON_CIPHER
    {"TLS_CLIENT_CERT_REQ_WITH_ANON_CIPHER", ERR_LIB_SSL, SSL_R_TLS_CLIENT_CERT_REQ_WITH_ANON_CIPHER},
  #else
    {"TLS_CLIENT_CERT_REQ_WITH_ANON_CIPHER", ERR_LIB_SSL, 232},
  #endif
  #ifdef SSL_R_TLS_HEARTBEAT_PEER_DOESNT_ACCEPT
    {"TLS_HEARTBEAT_PEER_DOESNT_ACCEPT", ERR_LIB_SSL, SSL_R_TLS_HEARTBEAT_PEER_DOESNT_ACCEPT},
  #else
    {"TLS_HEARTBEAT_PEER_DOESNT_ACCEPT", ERR_LIB_SSL, 365},
  #endif
  #ifdef SSL_R_TLS_HEARTBEAT_PENDING
    {"TLS_HEARTBEAT_PENDING", ERR_LIB_SSL, SSL_R_TLS_HEARTBEAT_PENDING},
  #else
    {"TLS_HEARTBEAT_PENDING", ERR_LIB_SSL, 366},
  #endif
  #ifdef SSL_R_TLS_ILLEGAL_EXPORTER_LABEL
    {"TLS_ILLEGAL_EXPORTER_LABEL", ERR_LIB_SSL, SSL_R_TLS_ILLEGAL_EXPORTER_LABEL},
  #else
    {"TLS_ILLEGAL_EXPORTER_LABEL", ERR_LIB_SSL, 367},
  #endif
  #ifdef SSL_R_TLS_INVALID_ECPOINTFORMAT_LIST
    {"TLS_INVALID_ECPOINTFORMAT_LIST", ERR_LIB_SSL, SSL_R_TLS_INVALID_ECPOINTFORMAT_LIST},
  #else
    {"TLS_INVALID_ECPOINTFORMAT_LIST", ERR_LIB_SSL, 157},
  #endif
  #ifdef SSL_R_TLS_PEER_DID_NOT_RESPOND_WITH_CERTIFICATE_LIST
    {"TLS_PEER_DID_NOT_RESPOND_WITH_CERTIFICATE_LIST", ERR_LIB_SSL, SSL_R_TLS_PEER_DID_NOT_RESPOND_WITH_CERTIFICATE_LIST},
  #else
    {"TLS_PEER_DID_NOT_RESPOND_WITH_CERTIFICATE_LIST", ERR_LIB_SSL, 233},
  #endif
  #ifdef SSL_R_TLS_RSA_ENCRYPTED_VALUE_LENGTH_IS_WRONG
    {"TLS_RSA_ENCRYPTED_VALUE_LENGTH_IS_WRONG", ERR_LIB_SSL, SSL_R_TLS_RSA_ENCRYPTED_VALUE_LENGTH_IS_WRONG},
  #else
    {"TLS_RSA_ENCRYPTED_VALUE_LENGTH_IS_WRONG", ERR_LIB_SSL, 234},
  #endif
  #ifdef SSL_R_TRIED_TO_USE_UNSUPPORTED_CIPHER
    {"TRIED_TO_USE_UNSUPPORTED_CIPHER", ERR_LIB_SSL, SSL_R_TRIED_TO_USE_UNSUPPORTED_CIPHER},
  #else
    {"TRIED_TO_USE_UNSUPPORTED_CIPHER", ERR_LIB_SSL, 235},
  #endif
  #ifdef SSL_R_UNABLE_TO_DECODE_DH_CERTS
    {"UNABLE_TO_DECODE_DH_CERTS", ERR_LIB_SSL, SSL_R_UNABLE_TO_DECODE_DH_CERTS},
  #else
    {"UNABLE_TO_DECODE_DH_CERTS", ERR_LIB_SSL, 236},
  #endif
  #ifdef SSL_R_UNABLE_TO_DECODE_ECDH_CERTS
    {"UNABLE_TO_DECODE_ECDH_CERTS", ERR_LIB_SSL, SSL_R_UNABLE_TO_DECODE_ECDH_CERTS},
  #else
    {"UNABLE_TO_DECODE_ECDH_CERTS", ERR_LIB_SSL, 313},
  #endif
  #ifdef SSL_R_UNABLE_TO_EXTRACT_PUBLIC_KEY
    {"UNABLE_TO_EXTRACT_PUBLIC_KEY", ERR_LIB_SSL, SSL_R_UNABLE_TO_EXTRACT_PUBLIC_KEY},
  #else
    {"UNABLE_TO_EXTRACT_PUBLIC_KEY", ERR_LIB_SSL, 237},
  #endif
  #ifdef SSL_R_UNABLE_TO_FIND_DH_PARAMETERS
    {"UNABLE_TO_FIND_DH_PARAMETERS", ERR_LIB_SSL, SSL_R_UNABLE_TO_FIND_DH_PARAMETERS},
  #else
    {"UNABLE_TO_FIND_DH_PARAMETERS", ERR_LIB_SSL, 238},
  #endif
  #ifdef SSL_R_UNABLE_TO_FIND_ECDH_PARAMETERS
    {"UNABLE_TO_FIND_ECDH_PARAMETERS", ERR_LIB_SSL, SSL_R_UNABLE_TO_FIND_ECDH_PARAMETERS},
  #else
    {"UNABLE_TO_FIND_ECDH_PARAMETERS", ERR_LIB_SSL, 314},
  #endif
  #ifdef SSL_R_UNABLE_TO_FIND_PUBLIC_KEY_PARAMETERS
    {"UNABLE_TO_FIND_PUBLIC_KEY_PARAMETERS", ERR_LIB_SSL, SSL_R_UNABLE_TO_FIND_PUBLIC_KEY_PARAMETERS},
  #else
    {"UNABLE_TO_FIND_PUBLIC_KEY_PARAMETERS", ERR_LIB_SSL, 239},
  #endif
  #ifdef SSL_R_UNABLE_TO_FIND_SSL_METHOD
    {"UNABLE_TO_FIND_SSL_METHOD", ERR_LIB_SSL, SSL_R_UNABLE_TO_FIND_SSL_METHOD},
  #else
    {"UNABLE_TO_FIND_SSL_METHOD", ERR_LIB_SSL, 240},
  #endif
  #ifdef SSL_R_UNABLE_TO_LOAD_SSL2_MD5_ROUTINES
    {"UNABLE_TO_LOAD_SSL2_MD5_ROUTINES", ERR_LIB_SSL, SSL_R_UNABLE_TO_LOAD_SSL2_MD5_ROUTINES},
  #else
    {"UNABLE_TO_LOAD_SSL2_MD5_ROUTINES", ERR_LIB_SSL, 241},
  #endif
  #ifdef SSL_R_UNABLE_TO_LOAD_SSL3_MD5_ROUTINES
    {"UNABLE_TO_LOAD_SSL3_MD5_ROUTINES", ERR_LIB_SSL, SSL_R_UNABLE_TO_LOAD_SSL3_MD5_ROUTINES},
  #else
    {"UNABLE_TO_LOAD_SSL3_MD5_ROUTINES", ERR_LIB_SSL, 242},
  #endif
  #ifdef SSL_R_UNABLE_TO_LOAD_SSL3_SHA1_ROUTINES
    {"UNABLE_TO_LOAD_SSL3_SHA1_ROUTINES", ERR_LIB_SSL, SSL_R_UNABLE_TO_LOAD_SSL3_SHA1_ROUTINES},
  #else
    {"UNABLE_TO_LOAD_SSL3_SHA1_ROUTINES", ERR_LIB_SSL, 243},
  #endif
  #ifdef SSL_R_UNEXPECTED_MESSAGE
    {"UNEXPECTED_MESSAGE", ERR_LIB_SSL, SSL_R_UNEXPECTED_MESSAGE},
  #else
    {"UNEXPECTED_MESSAGE", ERR_LIB_SSL, 244},
  #endif
  #ifdef SSL_R_UNEXPECTED_RECORD
    {"UNEXPECTED_RECORD", ERR_LIB_SSL, SSL_R_UNEXPECTED_RECORD},
  #else
    {"UNEXPECTED_RECORD", ERR_LIB_SSL, 245},
  #endif
  #ifdef SSL_R_UNINITIALIZED
    {"UNINITIALIZED", ERR_LIB_SSL, SSL_R_UNINITIALIZED},
  #else
    {"UNINITIALIZED", ERR_LIB_SSL, 276},
  #endif
  #ifdef SSL_R_UNKNOWN_ALERT_TYPE
    {"UNKNOWN_ALERT_TYPE", ERR_LIB_SSL, SSL_R_UNKNOWN_ALERT_TYPE},
  #else
    {"UNKNOWN_ALERT_TYPE", ERR_LIB_SSL, 246},
  #endif
  #ifdef SSL_R_UNKNOWN_CERTIFICATE_TYPE
    {"UNKNOWN_CERTIFICATE_TYPE", ERR_LIB_SSL, SSL_R_UNKNOWN_CERTIFICATE_TYPE},
  #else
    {"UNKNOWN_CERTIFICATE_TYPE", ERR_LIB_SSL, 247},
  #endif
  #ifdef SSL_R_UNKNOWN_CIPHER_RETURNED
    {"UNKNOWN_CIPHER_RETURNED", ERR_LIB_SSL, SSL_R_UNKNOWN_CIPHER_RETURNED},
  #else
    {"UNKNOWN_CIPHER_RETURNED", ERR_LIB_SSL, 248},
  #endif
  #ifdef SSL_R_UNKNOWN_CIPHER_TYPE
    {"UNKNOWN_CIPHER_TYPE", ERR_LIB_SSL, SSL_R_UNKNOWN_CIPHER_TYPE},
  #else
    {"UNKNOWN_CIPHER_TYPE", ERR_LIB_SSL, 249},
  #endif
  #ifdef SSL_R_UNKNOWN_CMD_NAME
    {"UNKNOWN_CMD_NAME", ERR_LIB_SSL, SSL_R_UNKNOWN_CMD_NAME},
  #else
    {"UNKNOWN_CMD_NAME", ERR_LIB_SSL, 386},
  #endif
  #ifdef SSL_R_UNKNOWN_DIGEST
    {"UNKNOWN_DIGEST", ERR_LIB_SSL, SSL_R_UNKNOWN_DIGEST},
  #else
    {"UNKNOWN_DIGEST", ERR_LIB_SSL, 368},
  #endif
  #ifdef SSL_R_UNKNOWN_KEY_EXCHANGE_TYPE
    {"UNKNOWN_KEY_EXCHANGE_TYPE", ERR_LIB_SSL, SSL_R_UNKNOWN_KEY_EXCHANGE_TYPE},
  #else
    {"UNKNOWN_KEY_EXCHANGE_TYPE", ERR_LIB_SSL, 250},
  #endif
  #ifdef SSL_R_UNKNOWN_PKEY_TYPE
    {"UNKNOWN_PKEY_TYPE", ERR_LIB_SSL, SSL_R_UNKNOWN_PKEY_TYPE},
  #else
    {"UNKNOWN_PKEY_TYPE", ERR_LIB_SSL, 251},
  #endif
  #ifdef SSL_R_UNKNOWN_PROTOCOL
    {"UNKNOWN_PROTOCOL", ERR_LIB_SSL, SSL_R_UNKNOWN_PROTOCOL},
  #else
    {"UNKNOWN_PROTOCOL", ERR_LIB_SSL, 252},
  #endif
  #ifdef SSL_R_UNKNOWN_REMOTE_ERROR_TYPE
    {"UNKNOWN_REMOTE_ERROR_TYPE", ERR_LIB_SSL, SSL_R_UNKNOWN_REMOTE_ERROR_TYPE},
  #else
    {"UNKNOWN_REMOTE_ERROR_TYPE", ERR_LIB_SSL, 253},
  #endif
  #ifdef SSL_R_UNKNOWN_SSL_VERSION
    {"UNKNOWN_SSL_VERSION", ERR_LIB_SSL, SSL_R_UNKNOWN_SSL_VERSION},
  #else
    {"UNKNOWN_SSL_VERSION", ERR_LIB_SSL, 254},
  #endif
  #ifdef SSL_R_UNKNOWN_STATE
    {"UNKNOWN_STATE", ERR_LIB_SSL, SSL_R_UNKNOWN_STATE},
  #else
    {"UNKNOWN_STATE", ERR_LIB_SSL, 255},
  #endif
  #ifdef SSL_R_UNSAFE_LEGACY_RENEGOTIATION_DISABLED
    {"UNSAFE_LEGACY_RENEGOTIATION_DISABLED", ERR_LIB_SSL, SSL_R_UNSAFE_LEGACY_RENEGOTIATION_DISABLED},
  #else
    {"UNSAFE_LEGACY_RENEGOTIATION_DISABLED", ERR_LIB_SSL, 338},
  #endif
  #ifdef SSL_R_UNSUPPORTED_CIPHER
    {"UNSUPPORTED_CIPHER", ERR_LIB_SSL, SSL_R_UNSUPPORTED_CIPHER},
  #else
    {"UNSUPPORTED_CIPHER", ERR_LIB_SSL, 256},
  #endif
  #ifdef SSL_R_UNSUPPORTED_COMPRESSION_ALGORITHM
    {"UNSUPPORTED_COMPRESSION_ALGORITHM", ERR_LIB_SSL, SSL_R_UNSUPPORTED_COMPRESSION_ALGORITHM},
  #else
    {"UNSUPPORTED_COMPRESSION_ALGORITHM", ERR_LIB_SSL, 257},
  #endif
  #ifdef SSL_R_UNSUPPORTED_DIGEST_TYPE
    {"UNSUPPORTED_DIGEST_TYPE", ERR_LIB_SSL, SSL_R_UNSUPPORTED_DIGEST_TYPE},
  #else
    {"UNSUPPORTED_DIGEST_TYPE", ERR_LIB_SSL, 326},
  #endif
  #ifdef SSL_R_UNSUPPORTED_ELLIPTIC_CURVE
    {"UNSUPPORTED_ELLIPTIC_CURVE", ERR_LIB_SSL, SSL_R_UNSUPPORTED_ELLIPTIC_CURVE},
  #else
    {"UNSUPPORTED_ELLIPTIC_CURVE", ERR_LIB_SSL, 315},
  #endif
  #ifdef SSL_R_UNSUPPORTED_PROTOCOL
    {"UNSUPPORTED_PROTOCOL", ERR_LIB_SSL, SSL_R_UNSUPPORTED_PROTOCOL},
  #else
    {"UNSUPPORTED_PROTOCOL", ERR_LIB_SSL, 258},
  #endif
  #ifdef SSL_R_UNSUPPORTED_SSL_VERSION
    {"UNSUPPORTED_SSL_VERSION", ERR_LIB_SSL, SSL_R_UNSUPPORTED_SSL_VERSION},
  #else
    {"UNSUPPORTED_SSL_VERSION", ERR_LIB_SSL, 259},
  #endif
  #ifdef SSL_R_UNSUPPORTED_STATUS_TYPE
    {"UNSUPPORTED_STATUS_TYPE", ERR_LIB_SSL, SSL_R_UNSUPPORTED_STATUS_TYPE},
  #else
    {"UNSUPPORTED_STATUS_TYPE", ERR_LIB_SSL, 329},
  #endif
  #ifdef SSL_R_USE_SRTP_NOT_NEGOTIATED
    {"USE_SRTP_NOT_NEGOTIATED", ERR_LIB_SSL, SSL_R_USE_SRTP_NOT_NEGOTIATED},
  #else
    {"USE_SRTP_NOT_NEGOTIATED", ERR_LIB_SSL, 369},
  #endif
  #ifdef SSL_R_VERSION_TOO_LOW
    {"VERSION_TOO_LOW", ERR_LIB_SSL, SSL_R_VERSION_TOO_LOW},
  #else
    {"VERSION_TOO_LOW", ERR_LIB_SSL, 396},
  #endif
  #ifdef SSL_R_WRITE_BIO_NOT_SET
    {"WRITE_BIO_NOT_SET", ERR_LIB_SSL, SSL_R_WRITE_BIO_NOT_SET},
  #else
    {"WRITE_BIO_NOT_SET", ERR_LIB_SSL, 260},
  #endif
  #ifdef SSL_R_WRONG_CERTIFICATE_TYPE
    {"WRONG_CERTIFICATE_TYPE", ERR_LIB_SSL, SSL_R_WRONG_CERTIFICATE_TYPE},
  #else
    {"WRONG_CERTIFICATE_TYPE", ERR_LIB_SSL, 383},
  #endif
  #ifdef SSL_R_WRONG_CIPHER_RETURNED
    {"WRONG_CIPHER_RETURNED", ERR_LIB_SSL, SSL_R_WRONG_CIPHER_RETURNED},
  #else
    {"WRONG_CIPHER_RETURNED", ERR_LIB_SSL, 261},
  #endif
  #ifdef SSL_R_WRONG_CURVE
    {"WRONG_CURVE", ERR_LIB_SSL, SSL_R_WRONG_CURVE},
  #else
    {"WRONG_CURVE", ERR_LIB_SSL, 378},
  #endif
  #ifdef SSL_R_WRONG_MESSAGE_TYPE
    {"WRONG_MESSAGE_TYPE", ERR_LIB_SSL, SSL_R_WRONG_MESSAGE_TYPE},
  #else
    {"WRONG_MESSAGE_TYPE", ERR_LIB_SSL, 262},
  #endif
  #ifdef SSL_R_WRONG_NUMBER_OF_KEY_BITS
    {"WRONG_NUMBER_OF_KEY_BITS", ERR_LIB_SSL, SSL_R_WRONG_NUMBER_OF_KEY_BITS},
  #else
    {"WRONG_NUMBER_OF_KEY_BITS", ERR_LIB_SSL, 263},
  #endif
  #ifdef SSL_R_WRONG_SIGNATURE_LENGTH
    {"WRONG_SIGNATURE_LENGTH", ERR_LIB_SSL, SSL_R_WRONG_SIGNATURE_LENGTH},
  #else
    {"WRONG_SIGNATURE_LENGTH", ERR_LIB_SSL, 264},
  #endif
  #ifdef SSL_R_WRONG_SIGNATURE_SIZE
    {"WRONG_SIGNATURE_SIZE", ERR_LIB_SSL, SSL_R_WRONG_SIGNATURE_SIZE},
  #else
    {"WRONG_SIGNATURE_SIZE", ERR_LIB_SSL, 265},
  #endif
  #ifdef SSL_R_WRONG_SIGNATURE_TYPE
    {"WRONG_SIGNATURE_TYPE", ERR_LIB_SSL, SSL_R_WRONG_SIGNATURE_TYPE},
  #else
    {"WRONG_SIGNATURE_TYPE", ERR_LIB_SSL, 370},
  #endif
  #ifdef SSL_R_WRONG_SSL_VERSION
    {"WRONG_SSL_VERSION", ERR_LIB_SSL, SSL_R_WRONG_SSL_VERSION},
  #else
    {"WRONG_SSL_VERSION", ERR_LIB_SSL, 266},
  #endif
  #ifdef SSL_R_WRONG_VERSION_NUMBER
    {"WRONG_VERSION_NUMBER", ERR_LIB_SSL, SSL_R_WRONG_VERSION_NUMBER},
  #else
    {"WRONG_VERSION_NUMBER", ERR_LIB_SSL, 267},
  #endif
  #ifdef SSL_R_X509_LIB
    {"X509_LIB", ERR_LIB_SSL, SSL_R_X509_LIB},
  #else
    {"X509_LIB", ERR_LIB_SSL, 268},
  #endif
  #ifdef SSL_R_X509_VERIFICATION_SETUP_PROBLEMS
    {"X509_VERIFICATION_SETUP_PROBLEMS", ERR_LIB_SSL, SSL_R_X509_VERIFICATION_SETUP_PROBLEMS},
  #else
    {"X509_VERIFICATION_SETUP_PROBLEMS", ERR_LIB_SSL, 269},
  #endif
  #ifdef X509_R_AKID_MISMATCH
    {"AKID_MISMATCH", ERR_LIB_X509, X509_R_AKID_MISMATCH},
  #else
    {"AKID_MISMATCH", ERR_LIB_X509, 110},
  #endif
  #ifdef X509_R_BAD_X509_FILETYPE
    {"BAD_X509_FILETYPE", ERR_LIB_X509, X509_R_BAD_X509_FILETYPE},
  #else
    {"BAD_X509_FILETYPE", ERR_LIB_X509, 100},
  #endif
  #ifdef X509_R_BASE64_DECODE_ERROR
    {"BASE64_DECODE_ERROR", ERR_LIB_X509, X509_R_BASE64_DECODE_ERROR},
  #else
    {"BASE64_DECODE_ERROR", ERR_LIB_X509, 118},
  #endif
  #ifdef X509_R_CANT_CHECK_DH_KEY
    {"CANT_CHECK_DH_KEY", ERR_LIB_X509, X509_R_CANT_CHECK_DH_KEY},
  #else
    {"CANT_CHECK_DH_KEY", ERR_LIB_X509, 114},
  #endif
  #ifdef X509_R_CERT_ALREADY_IN_HASH_TABLE
    {"CERT_ALREADY_IN_HASH_TABLE", ERR_LIB_X509, X509_R_CERT_ALREADY_IN_HASH_TABLE},
  #else
    {"CERT_ALREADY_IN_HASH_TABLE", ERR_LIB_X509, 101},
  #endif
  #ifdef X509_R_CRL_ALREADY_DELTA
    {"CRL_ALREADY_DELTA", ERR_LIB_X509, X509_R_CRL_ALREADY_DELTA},
  #else
    {"CRL_ALREADY_DELTA", ERR_LIB_X509, 127},
  #endif
  #ifdef X509_R_CRL_VERIFY_FAILURE
    {"CRL_VERIFY_FAILURE", ERR_LIB_X509, X509_R_CRL_VERIFY_FAILURE},
  #else
    {"CRL_VERIFY_FAILURE", ERR_LIB_X509, 131},
  #endif
  #ifdef X509_R_ERR_ASN1_LIB
    {"ERR_ASN1_LIB", ERR_LIB_X509, X509_R_ERR_ASN1_LIB},
  #else
    {"ERR_ASN1_LIB", ERR_LIB_X509, 102},
  #endif
  #ifdef X509_R_IDP_MISMATCH
    {"IDP_MISMATCH", ERR_LIB_X509, X509_R_IDP_MISMATCH},
  #else
    {"IDP_MISMATCH", ERR_LIB_X509, 128},
  #endif
  #ifdef X509_R_INVALID_DIRECTORY
    {"INVALID_DIRECTORY", ERR_LIB_X509, X509_R_INVALID_DIRECTORY},
  #else
    {"INVALID_DIRECTORY", ERR_LIB_X509, 113},
  #endif
  #ifdef X509_R_INVALID_FIELD_NAME
    {"INVALID_FIELD_NAME", ERR_LIB_X509, X509_R_INVALID_FIELD_NAME},
  #else
    {"INVALID_FIELD_NAME", ERR_LIB_X509, 119},
  #endif
  #ifdef X509_R_INVALID_TRUST
    {"INVALID_TRUST", ERR_LIB_X509, X509_R_INVALID_TRUST},
  #else
    {"INVALID_TRUST", ERR_LIB_X509, 123},
  #endif
  #ifdef X509_R_ISSUER_MISMATCH
    {"ISSUER_MISMATCH", ERR_LIB_X509, X509_R_ISSUER_MISMATCH},
  #else
    {"ISSUER_MISMATCH", ERR_LIB_X509, 129},
  #endif
  #ifdef X509_R_KEY_TYPE_MISMATCH
    {"KEY_TYPE_MISMATCH", ERR_LIB_X509, X509_R_KEY_TYPE_MISMATCH},
  #else
    {"KEY_TYPE_MISMATCH", ERR_LIB_X509, 115},
  #endif
  #ifdef X509_R_KEY_VALUES_MISMATCH
    {"KEY_VALUES_MISMATCH", ERR_LIB_X509, X509_R_KEY_VALUES_MISMATCH},
  #else
    {"KEY_VALUES_MISMATCH", ERR_LIB_X509, 116},
  #endif
  #ifdef X509_R_LOADING_CERT_DIR
    {"LOADING_CERT_DIR", ERR_LIB_X509, X509_R_LOADING_CERT_DIR},
  #else
    {"LOADING_CERT_DIR", ERR_LIB_X509, 103},
  #endif
  #ifdef X509_R_LOADING_DEFAULTS
    {"LOADING_DEFAULTS", ERR_LIB_X509, X509_R_LOADING_DEFAULTS},
  #else
    {"LOADING_DEFAULTS", ERR_LIB_X509, 104},
  #endif
  #ifdef X509_R_METHOD_NOT_SUPPORTED
    {"METHOD_NOT_SUPPORTED", ERR_LIB_X509, X509_R_METHOD_NOT_SUPPORTED},
  #else
    {"METHOD_NOT_SUPPORTED", ERR_LIB_X509, 124},
  #endif
  #ifdef X509_R_NEWER_CRL_NOT_NEWER
    {"NEWER_CRL_NOT_NEWER", ERR_LIB_X509, X509_R_NEWER_CRL_NOT_NEWER},
  #else
    {"NEWER_CRL_NOT_NEWER", ERR_LIB_X509, 132},
  #endif
  #ifdef X509_R_NO_CERT_SET_FOR_US_TO_VERIFY
    {"NO_CERT_SET_FOR_US_TO_VERIFY", ERR_LIB_X509, X509_R_NO_CERT_SET_FOR_US_TO_VERIFY},
  #else
    {"NO_CERT_SET_FOR_US_TO_VERIFY", ERR_LIB_X509, 105},
  #endif
  #ifdef X509_R_NO_CRL_NUMBER
    {"NO_CRL_NUMBER", ERR_LIB_X509, X509_R_NO_CRL_NUMBER},
  #else
    {"NO_CRL_NUMBER", ERR_LIB_X509, 130},
  #endif
  #ifdef X509_R_PUBLIC_KEY_DECODE_ERROR
    {"PUBLIC_KEY_DECODE_ERROR", ERR_LIB_X509, X509_R_PUBLIC_KEY_DECODE_ERROR},
  #else
    {"PUBLIC_KEY_DECODE_ERROR", ERR_LIB_X509, 125},
  #endif
  #ifdef X509_R_PUBLIC_KEY_ENCODE_ERROR
    {"PUBLIC_KEY_ENCODE_ERROR", ERR_LIB_X509, X509_R_PUBLIC_KEY_ENCODE_ERROR},
  #else
    {"PUBLIC_KEY_ENCODE_ERROR", ERR_LIB_X509, 126},
  #endif
  #ifdef X509_R_SHOULD_RETRY
    {"SHOULD_RETRY", ERR_LIB_X509, X509_R_SHOULD_RETRY},
  #else
    {"SHOULD_RETRY", ERR_LIB_X509, 106},
  #endif
  #ifdef X509_R_UNABLE_TO_FIND_PARAMETERS_IN_CHAIN
    {"UNABLE_TO_FIND_PARAMETERS_IN_CHAIN", ERR_LIB_X509, X509_R_UNABLE_TO_FIND_PARAMETERS_IN_CHAIN},
  #else
    {"UNABLE_TO_FIND_PARAMETERS_IN_CHAIN", ERR_LIB_X509, 107},
  #endif
  #ifdef X509_R_UNABLE_TO_GET_CERTS_PUBLIC_KEY
    {"UNABLE_TO_GET_CERTS_PUBLIC_KEY", ERR_LIB_X509, X509_R_UNABLE_TO_GET_CERTS_PUBLIC_KEY},
  #else
    {"UNABLE_TO_GET_CERTS_PUBLIC_KEY", ERR_LIB_X509, 108},
  #endif
  #ifdef X509_R_UNKNOWN_KEY_TYPE
    {"UNKNOWN_KEY_TYPE", ERR_LIB_X509, X509_R_UNKNOWN_KEY_TYPE},
  #else
    {"UNKNOWN_KEY_TYPE", ERR_LIB_X509, 117},
  #endif
  #ifdef X509_R_UNKNOWN_NID
    {"UNKNOWN_NID", ERR_LIB_X509, X509_R_UNKNOWN_NID},
  #else
    {"UNKNOWN_NID", ERR_LIB_X509, 109},
  #endif
  #ifdef X509_R_UNKNOWN_PURPOSE_ID
    {"UNKNOWN_PURPOSE_ID", ERR_LIB_X509, X509_R_UNKNOWN_PURPOSE_ID},
  #else
    {"UNKNOWN_PURPOSE_ID", ERR_LIB_X509, 121},
  #endif
  #ifdef X509_R_UNKNOWN_TRUST_ID
    {"UNKNOWN_TRUST_ID", ERR_LIB_X509, X509_R_UNKNOWN_TRUST_ID},
  #else
    {"UNKNOWN_TRUST_ID", ERR_LIB_X509, 120},
  #endif
  #ifdef X509_R_UNSUPPORTED_ALGORITHM
    {"UNSUPPORTED_ALGORITHM", ERR_LIB_X509, X509_R_UNSUPPORTED_ALGORITHM},
  #else
    {"UNSUPPORTED_ALGORITHM", ERR_LIB_X509, 111},
  #endif
  #ifdef X509_R_WRONG_LOOKUP_TYPE
    {"WRONG_LOOKUP_TYPE", ERR_LIB_X509, X509_R_WRONG_LOOKUP_TYPE},
  #else
    {"WRONG_LOOKUP_TYPE", ERR_LIB_X509, 112},
  #endif
  #ifdef X509_R_WRONG_TYPE
    {"WRONG_TYPE", ERR_LIB_X509, X509_R_WRONG_TYPE},
  #else
    {"WRONG_TYPE", ERR_LIB_X509, 122},
  #endif
    { NULL }
};
