#
# This file is for everybody to add tests for bugs that aren't
# fixed yet. Please add a test case and appropriate bug description.
#
# When you fix one of the bugs, please move the test to the correct
# test_ module.
#

import unittest
from test import support

#
# No test cases for outstanding bugs at the moment.
#


if __name__ == "__main__":
    unittest.main()
